import random
import time
from typing import Iterable

from . import config
from . import name_translation

class WgPeer:

    def __init__(self, json):
        self.online = None
        self.rx = None
        self.tx = None
        self.connection_json = None

        self.json = json

        self.uuid = json["uuid"]
        self.enabled = json["enabled"] == "1"
        self.ifname = json["servers"]
        self.natural_ifname = json["%servers"]
        self.name = json["name"]
        self.pubkey = json["pubkey"]

        self.tunnel_address = json["tunneladdress"]
        self.server_address = json["serveraddress"]
        self.server_port = json["serverport"]

        self.rx_speed = 0.0 # bits
        self.tx_speed = 0.0 # bits

        try:
            country = self.name.split("-")[0]
            city = self.name.split("-")[1]
            self.server = self.name.split("-")[2]

            try:
                self.country = name_translation.country_codes[country.upper()]
                self.city = name_translation.city_codes[city.upper()]

            except KeyError:
                self.country = country.upper()
                self.city = city.upper()

        except IndexError:
            self.country = "Unknown"
            self.city = "Unknown"
            self.server = self.name

    def add_connection_info(self, json) -> None:
        self.connection_json = json
        self.online = json["peer-status"] == "online" # ENABLED ONLY
        self.rx = json["transfer-rx"] # ENABLED ONLY
        self.tx = json["transfer-tx"] # ENABLED ONLY

    def is_target(self, target: str) -> bool:
        return self.uuid == target

    def get_human_info(self) -> str:
        rx = self.rx
        tx = self.tx
        if self.rx > 1000*1000*1000:
            rx = f"{"%.2f" % (self.rx / 1000 / 1000 / 1000)}GB"
        elif self.rx > 1000*1000:
            rx = f"{"%.2f" % (self.rx / 1000 / 1000)}MB"
        elif self.rx > 1000:
            rx = f"{"%.2f" % (self.rx / 1000)}KB"
        else:
            rx = f"{self.rx}B"

        if self.tx > 1000*1000*1000:
            tx = f"{"%.2f" % (self.tx / 1000 / 1000 / 1000)}GB"
        elif self.tx > 1000*1000:
            tx = f"{"%.2f" % (self.tx / 1000 / 1000)}MB"
        elif self.tx > 1000:
            tx = f"{"%.2f" % (self.tx / 1000)}KB"
        else:
            tx = f"{self.tx}B"

        return f"{self.name}\t({self.country},\t{self.city})\t[{rx}/{tx}]"


    def randomize_port(self, session, enable: int) -> None:

        port_range = random.choice(config.randomize_port_range)
        new_port = random.randint(port_range[0], port_range[1])

        request_json = {"client":
            {
                "enabled": enable,
                "name": self.name,
                "pubkey": self.pubkey,
                "psk": "",
                "tunneladdress": self.tunnel_address,
                "serveraddress": self.server_address,
                "serverport": new_port,
                "servers": config.instance_UUID,
                "keepalive": str(config.keepalive_interval),
            }
        }

        output = session.post(f"{config.opnsense_api}/wireguard/client/setClient/{self.uuid}", json=request_json, verify=False)

        if config.debug:
            print(output.content)

        self.server_port = new_port


    def enable(self, session, random_port = False) -> bool:
        if self.enabled:
            if not random_port:
                if config.verbose:
                    print(f"[!] {self.name} is already enabled.")
                return False
            else:
                print(f"[i] {self.name} is already enabled, shuffling ports.")

        if random_port:
            self.randomize_port(session, 1)
        else:
            output = session.post(f"{config.opnsense_api}/wireguard/client/toggleClient/{self.uuid}", verify=False)
            if config.debug:
                print(output.content)

        self.enabled = True
        return True

    def disable(self, session) -> bool:
        if not self.enabled:
            if config.verbose:
                print(f"[!] {self.name} is already disabled.")
            return False

        output = session.post(f"{config.opnsense_api}/wireguard/client/toggleClient/{self.uuid}", verify=False)
        if config.debug:
            print(output.content)

        self.enabled = False
        return True

    def set_speed(self, rx:float = None, tx:float = None) -> None:
        """
        Add speedtest record.
        :param rx: Download. In bits.
        :param tx: Upload. In bits.
        :return: None :3
        """
        if rx is not None:
            self.rx_speed = rx
        if tx is not None:
            self.tx_speed = tx



def connect_only_to(session, all_peers: Iterable[WgPeer], target: WgPeer) -> None:
    """
    Ensure all other peers are disabled except target, which will be ensured enabled.
    This method ensures no unnecessary API calls are made, by utilizing locally-cached enabled info.
    Dangerous if this becomes desynced from OPNSense.
    """
    # WgPeer already skips API calls when enabling/disabling state is already reached, so this method only skip the
    # Enabled->Disable->Enable again API calls for the target peer.

    for idx, peer in enumerate(all_peers):
        if peer != target:
            if config.verbose:
                print(f"Disabling {peer.name} (#{int(idx + 1)})...")
            peer.disable(session)

    print(f"Enabling {target.name}...")
    target.enable(session)

def wg_apply(session) -> None:
    output = session.post(f"{config.opnsense_api}/wireguard/service/reconfigure", verify=False)
    if config.debug:
        print(output.content)

    print("Waiting 1 second before checking status...")
    time.sleep(1)


import os
import sys

from . import cli

if not __package__:
    # Make CLI runnable from source tree with
    # python src/package
    package_source_path = os.path.dirname(os.path.dirname(__file__))
    sys.path.insert(0, package_source_path)

if __name__ == "__main__":
    cli.run()




# This file contains dictionaries that translate Mullvad's Country and City codes into full names.

# Country codes ( ISO 3166-1 alpha 2 )
country_codes = {
    "AF": "Afghanistan",
    "AX": "Aland Islands",
    "AL": "Albania",
    "DZ": "Algeria",
    "AS": "American Samoa",
    "AD": "Andorra",
    "AO": "Angola",
    "AI": "Anguilla",
    "AQ": "Antarctica",
    "AG": "Antigua And Barbuda",
    "AR": "Argentina",
    "AM": "Armenia",
    "AW": "Aruba",
    "AU": "Australia",
    "AT": "Austria",
    "AZ": "Azerbaijan",
    "BS": "Bahamas",
    "BH": "Bahrain",
    "BD": "Bangladesh",
    "BB": "Barbados",
    "BY": "Belarus",
    "BE": "Belgium",
    "BZ": "Belize",
    "BJ": "Benin",
    "BM": "Bermuda",
    "BT": "Bhutan",
    "BO": "Bolivia",
    "BA": "Bosnia And Herzegovina",
    "BW": "Botswana",
    "BV": "Bouvet Island",
    "BR": "Brazil",
    "IO": "British Indian Ocean Territory",
    "BN": "Brunei Darussalam",
    "BG": "Bulgaria",
    "BF": "Burkina Faso",
    "BI": "Burundi",
    "KH": "Cambodia",
    "CM": "Cameroon",
    "CA": "Canada",
    "CV": "Cape Verde",
    "KY": "Cayman Islands",
    "CF": "Central African Republic",
    "TD": "Chad",
    "CL": "Chile",
    "CN": "China",
    "CX": "Christmas Island",
    "CC": "Cocos (Keeling) Islands",
    "CO": "Colombia",
    "KM": "Comoros",
    "CG": "Congo",
    "CD": "Congo, Democratic Republic",
    "CK": "Cook Islands",
    "CR": "Costa Rica",
    "CI": "Cote D\"Ivoire",
    "HR": "Croatia",
    "CU": "Cuba",
    "CY": "Cyprus",
    "CZ": "Czech Republic",
    "DK": "Denmark",
    "DJ": "Djibouti",
    "DM": "Dominica",
    "DO": "Dominican Republic",
    "EC": "Ecuador",
    "EG": "Egypt",
    "SV": "El Salvador",
    "GQ": "Equatorial Guinea",
    "ER": "Eritrea",
    "EE": "Estonia",
    "ET": "Ethiopia",
    "FK": "Falkland Islands (Malvinas)",
    "FO": "Faroe Islands",
    "FJ": "Fiji",
    "FI": "Finland",
    "FR": "France",
    "GF": "French Guiana",
    "PF": "French Polynesia",
    "TF": "French Southern Territories",
    "GA": "Gabon",
    "GM": "Gambia",
    "GE": "Georgia",
    "DE": "Germany",
    "GH": "Ghana",
    "GI": "Gibraltar",
    "GR": "Greece",
    "GL": "Greenland",
    "GD": "Grenada",
    "GP": "Guadeloupe",
    "GU": "Guam",
    "GT": "Guatemala",
    "GG": "Guernsey",
    "GN": "Guinea",
    "GW": "Guinea-Bissau",
    "GY": "Guyana",
    "HT": "Haiti",
    "HM": "Heard Island & Mcdonald Islands",
    "VA": "Holy See (Vatican City State)",
    "HN": "Honduras",
    "HK": "Hong Kong",
    "HU": "Hungary",
    "IS": "Iceland",
    "IN": "India",
    "ID": "Indonesia",
    "IR": "Iran, Islamic Republic Of",
    "IQ": "Iraq",
    "IE": "Ireland",
    "IM": "Isle Of Man",
    "IL": "Israel",
    "IT": "Italy",
    "JM": "Jamaica",
    "JP": "Japan",
    "JE": "Jersey",
    "JO": "Jordan",
    "KZ": "Kazakhstan",
    "KE": "Kenya",
    "KI": "Kiribati",
    "KR": "Korea",
    "KP": "North Korea",
    "KW": "Kuwait",
    "KG": "Kyrgyzstan",
    "LA": "Lao People\"s Democratic Republic",
    "LV": "Latvia",
    "LB": "Lebanon",
    "LS": "Lesotho",
    "LR": "Liberia",
    "LY": "Libyan Arab Jamahiriya",
    "LI": "Liechtenstein",
    "LT": "Lithuania",
    "LU": "Luxembourg",
    "MO": "Macao",
    "MK": "Macedonia",
    "MG": "Madagascar",
    "MW": "Malawi",
    "MY": "Malaysia",
    "MV": "Maldives",
    "ML": "Mali",
    "MT": "Malta",
    "MH": "Marshall Islands",
    "MQ": "Martinique",
    "MR": "Mauritania",
    "MU": "Mauritius",
    "YT": "Mayotte",
    "MX": "Mexico",
    "FM": "Micronesia, Federated States Of",
    "MD": "Moldova",
    "MC": "Monaco",
    "MN": "Mongolia",
    "ME": "Montenegro",
    "MS": "Montserrat",
    "MA": "Morocco",
    "MZ": "Mozambique",
    "MM": "Myanmar",
    "NA": "Namibia",
    "NR": "Nauru",
    "NP": "Nepal",
    "NL": "Netherlands",
    "AN": "Netherlands Antilles",
    "NC": "New Caledonia",
    "NZ": "New Zealand",
    "NI": "Nicaragua",
    "NE": "Niger",
    "NG": "Nigeria",
    "NU": "Niue",
    "NF": "Norfolk Island",
    "MP": "Northern Mariana Islands",
    "NO": "Norway",
    "OM": "Oman",
    "PK": "Pakistan",
    "PW": "Palau",
    "PS": "Palestinian Territory, Occupied",
    "PA": "Panama",
    "PG": "Papua New Guinea",
    "PY": "Paraguay",
    "PE": "Peru",
    "PH": "Philippines",
    "PN": "Pitcairn",
    "PL": "Poland",
    "PT": "Portugal",
    "PR": "Puerto Rico",
    "QA": "Qatar",
    "RE": "Reunion",
    "RO": "Romania",
    "RU": "Russian Federation",
    "RW": "Rwanda",
    "BL": "Saint Barthelemy",
    "SH": "Saint Helena",
    "KN": "Saint Kitts And Nevis",
    "LC": "Saint Lucia",
    "MF": "Saint Martin",
    "PM": "Saint Pierre And Miquelon",
    "VC": "Saint Vincent And Grenadines",
    "WS": "Samoa",
    "SM": "San Marino",
    "ST": "Sao Tome And Principe",
    "SA": "Saudi Arabia",
    "SN": "Senegal",
    "RS": "Serbia",
    "SC": "Seychelles",
    "SL": "Sierra Leone",
    "SG": "Singapore",
    "SK": "Slovakia",
    "SI": "Slovenia",
    "SB": "Solomon Islands",
    "SO": "Somalia",
    "ZA": "South Africa",
    "GS": "South Georgia And Sandwich Isl.",
    "ES": "Spain",
    "LK": "Sri Lanka",
    "SD": "Sudan",
    "SR": "Suriname",
    "SJ": "Svalbard And Jan Mayen",
    "SZ": "Swaziland",
    "SE": "Sweden",
    "CH": "Switzerland",
    "SY": "Syrian Arab Republic",
    "TW": "Taiwan",
    "TJ": "Tajikistan",
    "TZ": "Tanzania",
    "TH": "Thailand",
    "TL": "Timor-Leste",
    "TG": "Togo",
    "TK": "Tokelau",
    "TO": "Tonga",
    "TT": "Trinidad And Tobago",
    "TN": "Tunisia",
    "TR": "Turkey",
    "TM": "Turkmenistan",
    "TC": "Turks And Caicos Islands",
    "TV": "Tuvalu",
    "UG": "Uganda",
    "UA": "Ukraine",
    "AE": "United Arab Emirates",
    "GB": "United Kingdom",
    "US": "United States",
    "UM": "United States Outlying Islands",
    "UY": "Uruguay",
    "UZ": "Uzbekistan",
    "VU": "Vanuatu",
    "VE": "Venezuela",
    "VN": "Vietnam",
    "VG": "Virgin Islands, British",
    "VI": "Virgin Islands, U.S.",
    "WF": "Wallis And Futuna",
    "EH": "Western Sahara",
    "YE": "Yemen",
    "ZM": "Zambia",
    "ZW": "Zimbabwe"
}

# City codes (IATA)
city_codes = {
    "AAA": "Taegu",
    "AAE": "Annaba",
    "ABB": "Lac Abbe",
    "ABE": "Allentown",
    "ABI": "Abilene",
    "ABJ": "Abidjan",
    "ABL": "Ambler",
    "ABM": "Bamaga",
    "ABQ": "Albuquerque",
    "ABR": "Aberdeen Wa",
    "ABS": "Abu Simbel",
    "ABX": "Albury",
    "ABY": "Abyaneh",
    "ABZ": "Aberdeen",
    "AC1": "Soppero",
    "ACA": "Acapulco",
    "ACC": "Accra",
    "ACD": "Acandi",
    "ACE": "Tenerife",
    "ACI": "Alderney",
    "ACK": "Nantucket",
    "ACN": "Arches & Canyonlands",
    "ACR": "Araracuara",
    "ACT": "Waco",
    "ACV": "Arcata",
    "ACY": "Atlantic City /Atlantic Cty",
    "ADA": "Adana",
    "ADA1": "CilvegšzŸ/Bab Elhawa Border",
    "ADB": "Izmir",
    "ADD": "Addis Ababa",
    "ADD1": "Langano",
    "ADD6": "Sidama",
    "ADE": "Aden",
    "ADL": "Adelaide",
    "ADQ": "Kodiak",
    "ADR": "Angra Dos Reis",
    "ADU": "Ardebil",
    "ADZ": "San Andres",
    "AEP": "Aeroparque Airport",
    "AES": "Aalesund",
    "AET": "Allakaket",
    "AEY": "Akureyri",
    "AF1": "The Band-i-Amir",
    "AF2": "Balkh",
    "AFA": "San Rafael Md",
    "AGA": "Agadir",
    "AGB": "Augsburg",
    "AGC": "Aguas Calientes",
    "AGF": "Agen",
    "AGH": "Aghadowey",
    "AGI": "Arroyo Grande Ca",
    "AGP": "Malaga",
    "AGR": "Agra",
    "AGR1": "Trichy",
    "AGR2": "Bandipur",
    "AGR4": "Kabini ",
    "AGU": "Aguascalientes",
    "AGV": "Acarigua",
    "AGX": "Agatti Island",
    "AHF": "Ashford, Wa",
    "AHO": "Alghero",
    "AHU": "Ashu ",
    "AIA": "Alliance",
    "AIN": "Wainwright",
    "AIR": "Aripuana",
    "AIS": "Aisen Region",
    "AIY": "Atlantic City",
    "AJA": "Ajaccio",
    "AJL": "Aizawl",
    "AJU": "Aracaju ",
    "AJY": "Agadez",
    "AKD": "Ancud",
    "AKJ": "Asahikawa",
    "AKL": "Auckland",
    "AKS": "Aksu",
    "AKY": "Sittwe",
    "AL1": "Kruja",
    "ALA": "Almaty",
    "ALA1": "Kolsai Lakes",
    "ALA2": "Taraz",
    "ALA3": "Aksu Djabagly Reserve",
    "ALA4": "Tulkubas",
    "ALA5": "Zharkent",
    "ALA6": "Charyn Canyon",
    "ALA7": "Karkara",
    "ALB": "Albany",
    "ALC": "Alicante",
    "ALE": "Alexandria, La",
    "ALF": "Alta",
    "ALG": "Algiers",
    "ALM": "Alamogordo Nm",
    "ALO": "Waterloo",
    "ALP": "Aleppo",
    "ALP1": "Alpine",
    "ALP2": "The Euphrates",
    "ALR": "Almora",
    "ALS": "Alamosa",
    "ALW": "Walla Walla",
    "ALX": "Alexandria, Va",
    "ALY": "Alexandria",
    "AMA": "Amarillo",
    "AMB": "Ambato",
    "AMBR": "Amazon Brazil",
    "AMD": "Ahmedabad",
    "AMEC": "Amazon Ecuador",
    "AMH": "Arba Minch",
    "AMI": "Mataram",
    "AML": "Amla",
    "AMM": "Amman",
    "AMM1": "Petra",
    "AMM2": "Dead Sea ",
    "AMM3": "Wadi Rum",
    "AMM4": "Dana Nature Reserve",
    "AMM5": "Irbid",
    "AMM6": "Nasib/Jaber Border",
    "AMM7": "Feinan",
    "AMM8": "Jerash",
    "AMM9": "Kerak",
    "AMP": "Amarapura",
    "AMPE": "Amazon Peru",
    "AMQ": "Ambon",
    "AMR": "Amarillo, Tx",
    "AMS": "Amsterdam",
    "AMZ": "Amazon",
    "ANA": "Anaheim",
    "ANB": "Anniston",
    "ANC": "Anchorage",
    "ANE": "Anacortes",
    "ANF": "Antofagasta",
    "ANG": "Anegundi",
    "ANI": "Aniak",
    "ANJ": "Anjouan",
    "ANK": "Ankara",
    "ANN": "Annapolis, Md",
    "ANP": "Amboro National Park",
    "ANR": "Antwerp",
    "ANS": "Andahuaylas",
    "ANT": "Antigua",
    "ANU": "Antigua",
    "ANV": "Anvik",
    "AOB": "Arenal",
    "AOI": "Ancona",
    "AOJ": "Aomori",
    "AOK": "Karpathos",
    "AOO": "Altoona / Martinsburg",
    "AOR": "Alor Setar",
    "APC": "Napa Valley",
    "APF": "Naples",
    "APL": "Nampula",
    "APN": "Alpena",
    "APO": "Apartado",
    "APW": "Apia",
    "APY": "Alleppey",
    "APY2": "Kumbakonam",
    "APZ": "Zapala Ne",
    "AQJ": "Aqaba",
    "AQJ1": "Azraq",
    "AQP": "Arequipa",
    "AR2": "Tierra del Fuego",
    "AR4": "Mount FitzRoy",
    "ARH": "Arkhangelsk",
    "ARI": "Arica",
    "ARK": "Arusha",
    "ARM": "Armidale",
    "ARN": "Stockholm",
    "ARR": "Alto Rio Senguerr",
    "ART": "Watertown",
    "ARU": "Aracatuba ",
    "ASB": "Ashgabat",
    "ASB2": "Darvaza",
    "ASB3": "Farab Border",
    "ASB4": "Mashhad",
    "ASB5": "Balkanabat",
    "ASD": "Andros Town",
    "ASL": "Lac Assal",
    "ASM": "Asmara",
    "ASP": "Aspen",
    "ASR": "Kayseri",
    "ASR1": "Urgup (Cappadocia)",
    "AST": "Astoria",
    "ASU": "Asuncion",
    "ASW": "Aswan",
    "ASW1": "Awash National Park",
    "ATA": "Anta",
    "ATB": "Atlantic Beach Nc",
    "ATC": "Arthurs Town",
    "ATH": "Athens",
    "ATL": "Atlanta",
    "ATM": "Atacama Desert",
    "ATQ": "Amritsar",
    "ATW": "Appleton",
    "ATY": "Watertown",
    "ATZ": "Assiut",
    "AUA": "Aruba",
    "AUC": "Arauca",
    "AUG": "Augusta",
    "AUH": "Abu Dhabi",
    "AUS": "Austin",
    "AUX": "Araguaiana",
    "AVE": "Puerto Aventuras",
    "AVI": "Ciego De Avila",
    "AVK": "Karakorum",
    "AVK1": "Ugii Lake",
    "AVL": "Asheville / Hendersonville",
    "AVN": "Avignon",
    "AVP": "Wilkes Barre/Scranton",
    "AWZ": "Ahwaz",
    "AWZ1": "Andimeshk",
    "AXA": "Anguilla",
    "AXM": "Armenia",
    "AXU": "Axum",
    "AXU1": "Mekele",
    "AYG": "Yaguara",
    "AYP": "Ayacucho",
    "AYQ": "Ayers Rock",
    "AYR": "Ayr Scotland",
    "AYT": "Antalya",
    "AZD": "Yazd",
    "AZD1": "Zein al-Din Caravanserai",
    "AZN": "Andizhan",
    "AZO": "Kalamazoo",
    "BAB": "Baalbek",
    "BAB1": "Sidon and Tyre",
    "BAG": "Bagan",
    "BAH": "Bahrain",
    "BAK": "Baku",
    "BAK1": "Sheki",
    "BAK2": "Kish",
    "BAK3": "Shamakhi",
    "BAK4": "Gabala",
    "BAK5": "Lagodekhi Border",
    "BAK6": "Lenkoran",
    "BAK7": "Astara Border",
    "BAK8": "Guba",
    "BAK9": "Pirguli",
    "BAL": "Ballygally",
    "BAN": "Ba–os",
    "BAQ": "Barranquilla",
    "BAR": "Bargtang Valley",
    "BAS": "Bass Lake, Ca",
    "BAT": "Baltra",
    "BAU": "Bauru",
    "BAV": "Bavanat Valley",
    "BAX": "Barnaul",
    "BAZ": "Bazaruto Archipelago",
    "BBA": "Balmaceda",
    "BBI": "Bhubaneswar",
    "BBK": "Kasane",
    "BBK1": "Nalapak Village",
    "BBM": "Battambang",
    "BBN": "Bario",
    "BBN1": "Bario Highlands Camp 1",
    "BBN3": "Batu Lawih",
    "BBU": "Bucharest",
    "BCA": "Baracoa",
    "BCD": "Bacolod",
    "BCL": "Barra Colorado",
    "BCN": "Barcelona",
    "BCO": "Jinka",
    "BCO1": "Mago National Park",
    "BCO2": "Turmi",
    "BCO3": "Konso",
    "BDA": "Bermuda",
    "BDH": "Bodhgaya",
    "BDJ": "Banjarmasin",
    "BDL": "Hartford",
    "BDO": "Bandung",
    "BDQ": "Vadodara",
    "BDR": "Bridgeport",
    "BED": "Bedford",
    "BEF": "Bluefields",
    "BEH": "Benton Harbor",
    "BEI": "Beica",
    "BEK": "Rae Bareli",
    "BEL": "Belem",
    "BEN": "Benghazi",
    "BEN1": "Cyrene & Apollonia ",
    "BEP": "Bellary",
    "BER": "Berlin",
    "BET": "Bethel",
    "BEV": "Beverly Hills",
    "BEW": "Beira",
    "BEY": "Beirut",
    "BEY1": "Zahle",
    "BEY2": "Bcharre",
    "BEY3": "Kfardebiane",
    "BEY4": "Byblos",
    "BEY5": "Aanjar",
    "BEY6": "Masnaa Border",
    "BFD": "Bradford",
    "BFF": "Scottsbluff",
    "BFN": "Bloemfontein",
    "BFS": "Belfast",
    "BGA": "Bucaramanga",
    "BGF": "Bangui",
    "BGI": "Bridgetown",
    "BGK": "Big Creek",
    "BGL": "Ballygally",
    "BGM": "Binghampton",
    "BGO": "Bergen",
    "BGR": "Bangor",
    "BH": "Belo Horizonte",
    "BHA": "Bahia De Caraquez",
    "BHA1": "Bhainsrorgarh",
    "BHB": "Bar Harbor",
    "BHE": "Bhenswara",
    "BHI": "Bahia Blanca ",
    "BHJ": "Bhuj ",
    "BHK": "Bukhara",
    "BHK1": "Shakrisabz",
    "BHM": "Birmingham, Al",
    "BHO": "Bhopal",
    "BHQ": "Broken Hill",
    "BHR": "Bharatpur",
    "BHU": "Bhavnagar Civil",
    "BHU1": "Trashigang",
    "BHX": "Birmingham",
    "BHZ": "Belo Horizonte",
    "BIA": "Bastia",
    "BIG": "Big Bend National Pk",
    "BIJ": "Bijaipur",
    "BIK": "Biak",
    "BIL": "Billings",
    "BIM": "Bimini",
    "BIN": "Bamiyan",
    "BIO": "Bilbao",
    "BIQ": "Biarritz",
    "BIS": "Bishop",
    "BJ": "Bhuj",
    "BJI": "Bemidji",
    "BJL": "Banjul",
    "BJP": "Bijapur",
    "BJR": "Bahar Dar",
    "BJT": "Bentota",
    "BJX": "Leon- Guanajuato",
    "BK11": "Mount Kinabalu National Park",
    "BKB": "Bikaner",
    "BKI": "Kota Kinabalu",
    "BKK": "Bangkok",
    "BKK1": "Ayutthaya",
    "BKK2": "Kanchanaburi",
    "BKK3": "Koh Samet",
    "BKN": "Birni Nkoni",
    "BKO": "Bamako",
    "BKS": "Bakersfield",
    "BKW": "Beckley",
    "BKX": "Brookings",
    "BLA": "Barcelona",
    "BLF": "Bluefield",
    "BLI": "Bellingham",
    "BLK": "Blackpool",
    "BLL": "Billund",
    "BLP": "Bellavista",
    "BLQ": "Bologna",
    "BLR": "Bangalore",
    "BLX": "Biloxi, Ms",
    "BLY": "Blythe California",
    "BLZ": "Blantyre",
    "BME": "Bangor Maine",
    "BMG": "Bloomington",
    "BMI": "Bloomington",
    "BMT": "Bumthang",
    "BMV": "Buon Me Thuot",
    "BNA": "Nashville",
    "BNE": "Brisbane",
    "BNJ": "Bonn",
    "BNN": "Bronnoysund",
    "BNN2": "Bario Highlands Camp 2",
    "BNN4": "Pa Ukat",
    "BNP": "Bandhavgarh National Park",
    "BNS": "Barinas",
    "BNT": "Bundi",
    "BNY": "Burnley",
    "BOC": "Boca Raton",
    "BOD": "Bordeaux",
    "BOG": "Bogota",
    "BOH": "Bournemouth",
    "BOI": "Boise",
    "BOM": "Mumbai",
    "BON": "Bonaire",
    "BOO": "Bodo",
    "BOS": "Boston",
    "BOY": "Bobo Dioulasso",
    "BP": "Bonampak",
    "BPD": "Boipeba Island",
    "BPG": "Barra Do Garcas ",
    "BPS": "Porto Seguro",
    "BPT": "Beaumont",
    "BPU": "Beppu",
    "BQK": "Brunswick",
    "BQN": "Aguadilla",
    "BRA": "Barreiras",
    "BRC": "Bariloche ",
    "BRD": "Brainerd",
    "BRE": "Bremen",
    "BRI": "Bari",
    "BRM": "Barquisimeto",
    "BRN": "Berne",
    "BRO": "Brownsville",
    "BRQ": "Brno",
    "BRR": "Barra / Hebrides Islands",
    "BRS": "Bristol",
    "BRT": "Bathurst Island",
    "BRU": "Brussels",
    "BRW": "Barrow",
    "BRY": "Bryce Canyon Nat Pk",
    "BSB": "Brasilia",
    "BSK": "Biskra",
    "BSL": "Basel",
    "BSS": "Balsas ",
    "BTM": "Butte",
    "BTN": "Boston Lincolnshire",
    "BTR": "Baton Rouge",
    "BTS": "Bratislava",
    "BTU": "Bintulu",
    "BTV": "Burlington",
    "BUD": "Budapest",
    "BUE": "Buenos Aires",
    "BUF": "Buffalo",
    "BUL1": "Bulgan",
    "BUN": "Bunratty",
    "BUP": "Bhatinda",
    "BUQ": "Bulawayo",
    "BUR": "Burlington",
    "BUS": "Batumi",
    "BUS1": "Sarpi Border",
    "BUS2": "Gobustan",
    "BUT": "Buttla",
    "BUZ": "Bushehr",
    "BVA": "Bellavista Cloud Forest",
    "BVB": "Boa Vista ",
    "BVE": "Brive-La-Gaillarde",
    "BVH": "Vilhena Ro",
    "BVI": "Birdsville",
    "BVM": "Belmonte ",
    "BVR": "Budva Riviera ",
    "BWA": "Bhairawa",
    "BWI": "Baltimore",
    "BWN": "Bandar Seri Begawan",
    "BXN": "Bodrum",
    "BXU": "Butuan",
    "BYM": "Bayamo",
    "BYU": "Bayreuth",
    "BZC": "Buzios",
    "BZE": "Belize City",
    "BZN": "Bozeman",
    "BZR": "Beziers",
    "BZRF": "Belize Rain Forest",
    "CA": "Cuddles",
    "CAC": "Cascavel ",
    "CAE": "Carlisle",
    "CAG": "Cagliari",
    "CAH": "Cachi",
    "CAI": "Cairo",
    "CAI1": "Minya",
    "CAI2": "Bahariya Oasis ",
    "CAJ": "Canaima",
    "CAK": "Akron/Canton",
    "CAL": "Caldas Novas",
    "CAM": "Cambria",
    "CAN": "Guangzhou",
    "CAP": "Captiva Island",
    "CAQ": "Caucasia",
    "CAR": "Carrickfergus",
    "CAS": "Casper Wy",
    "CAT": "Chichicastenango",
    "CAV": "Co.Cavan",
    "CAW": "Campos ",
    "CAY": "Cayenne",
    "CBB": "Cochabamba",
    "CBD": "Carlsbad Nm",
    "CBE": "Cumberland",
    "CBG": "Cambridge",
    "CBL": "Ciudad Bolivar",
    "CBN": "Coban",
    "CBR": "Canberra",
    "CBS": "Cabo San Lucas",
    "CBU": "Corumbau",
    "CCC": "Caleta Tortel",
    "CCF": "Carcassonne",
    "CCH": "Colchagua Valley",
    "CCJ": "Calicut",
    "CCM": "Criciuma ",
    "CCN": "Caracautin",
    "CCO": "Cerro Castillo",
    "CCP": "Concepcion",
    "CCR": "Cancha Carrera",
    "CCS": "Caracas",
    "CCT": "Colonia Catriel Rn",
    "CCT1": "Calchaqul Valley",
    "CCU": "Calcutta",
    "CCV": "Corcovado",
    "CDB": "Cuddapah",
    "CDC": "Canyon De Chelly Nat",
    "CDG": "Chapada dos Guimaraes",
    "CDH": "Camden",
    "CDJ": "Conceicao/Araguaia",
    "CDO": "Castle Douglas",
    "CDR": "Chadron",
    "CDV": "Cordova",
    "CDY": "Cody Wy",
    "CEB": "Cebu",
    "CEC": "Crescent City",
    "CEI": "Chiang Rai",
    "CEN": "Ciudad Obregon",
    "CENH": "Central Highlands",
    "CER": "Cherbourg",
    "CEZ": "Cortez",
    "CFE": "Clermont Ferrands",
    "CFG": "Cienfuegos",
    "CFN": "Donegal",
    "CFO": "Confreza ",
    "CFR": "Caen",
    "CFU": "Kerkyra",
    "CFY": "Cafayate",
    "CG1": "Kisangani",
    "CG2": "Kinshasa",
    "CG3": "The Congo River",
    "CGA": "Craig",
    "CGB": "Cuiaba ",
    "CGI": "Cape Girardeau",
    "CGN": "Cologne",
    "CGO": "Zhengzhou",
    "CGO1": "Kaifeng",
    "CGP": "Chittagong",
    "CGR": "Campo Grande",
    "CGU": "Ciudad Guayana",
    "CHA": "Charlottesville",
    "CHBR": "Chobe River Region",
    "CHC": "Christchurch",
    "CHE": "Cheyenne",
    "CHH": "Chachapoyas",
    "CHI": "Chicago",
    "CHK": "Cherokee",
    "CHL": "Chiloe",
    "CHM": "Chimbote",
    "CHO": "Cholpon Ata",
    "CHP": "Cholpon Ata",
    "CHQ": "Chania",
    "CHR": "Chaa Creek",
    "CHS": "Charleston-Sth Carol",
    "CHT": "Chittorgarh",
    "CHU": "Chugchilan ",
    "CIC": "Chico",
    "CID": "Cedar Rapids",
    "CIS": "Catalina Island",
    "CIT": "Chimkent",
    "CIT1": "Turkistan",
    "CIU": "Sault Ste Marie",
    "CIX": "Chiclayo",
    "CJA": "Cajamarca",
    "CJB": "Coimbatore",
    "CJC": "Calama",
    "CJD": "Candilejas",
    "CJL": "Chitral",
    "CJS": "Ciudad Juarez",
    "CJU": "Jeju Island ",
    "CKB": "Clarksburg",
    "CKG": "Chongqing",
    "CKS": "Carajas ",
    "CKY": "Conakry",
    "CLA": "Co.Clare",
    "CLB": "Columbia",
    "CLC": "Colca Canyon",
    "CLD": "Carlsbad",
    "CLE": "Cleveland",
    "CLJ": "Cluj",
    "CLL": "College Station",
    "CLN": "El Chalten",
    "CLO": "Cali",
    "CLQ": "Colima",
    "CLS": "Calais",
    "CLT": "Charlotte",
    "CLW": "Clearwater",
    "CLY": "Calvi",
    "CMB": "Colombo",
    "CMB1": "Negombo",
    "CMB2": "Anuradhapura",
    "CMB3": "Polonnaruwa",
    "CMB4": "Yala",
    "CMB5": "Dambulla",
    "CMB6": "Tea Trails",
    "CMB7": "Sinharaja Rain Forest",
    "CMB8": "Adams Peak",
    "CMB9": "Tangelle",
    "CME": "Ciudad Del Carmen",
    "CMF": "Chambery",
    "CMG": "Corumba",
    "CMH": "Columbus",
    "CMI": "Champaign",
    "CMN": "Casablanca",
    "CMP": "Santana Do Araguaia",
    "CMW": "Camaguey",
    "CMX": "Hancock",
    "CN1": "Yunnan",
    "CND": "Constanta",
    "CNF": "Belo Horizonte /Belo Horizon",
    "CNM": "Carlsbad",
    "CNN": "Chicanna",
    "CNQ": "Corrientes",
    "CNS": "Cairns",
    "CNX": "Chiang Mai",
    "CNY": "Moab",
    "CO1": "San Gil",
    "CO2": "Los Nevados National Park",
    "CO3": "Los LLanos",
    "COC": "Cocoa Beach",
    "COH": "Cooch Behar",
    "COK": "Cochin",
    "COL": "Colorado Springs",
    "CON": "Corning",
    "COO": "Cotonou",
    "COR": "Cordoba",
    "COZ": "Coffee Zone",
    "CPA": "Copacabana",
    "CPB": "Capurgana",
    "CPC": "San Martin de los Andes",
    "CPE": "Campeche",
    "CPH": "Copenhagen",
    "CPO": "Copiapo",
    "CPQ": "Campinas",
    "CPR": "Casper",
    "CPT": "Cape Town",
    "CPV": "Campina Grande ",
    "CQT": "Caquetania",
    "CRB": "Corbett",
    "CRD": "Comodoro Rivadavia",
    "CRI": "Crooked Island",
    "CRN": "Choroni",
    "CRP": "Corpus-Christi",
    "CRQ": "Caravelas ",
    "CRR": "Carara",
    "CRU": "Carriacou Island",
    "CRW": "Charleston",
    "CRZ": "Turkmenabat",
    "CST": "Caprivi Strip",
    "CSX": "Changsha",
    "CTA": "Catania",
    "CTF": "Central Farm",
    "CTG": "Cartagena de Indias",
    "CTJ": "Cetinje ",
    "CTL": "Charleville",
    "CTM": "Chetumal",
    "CTN": "Chattanooga",
    "CTU": "Chengdu",
    "CTU1": "Emeishan",
    "CTU2": "Wolong Nature Reserve",
    "CUC": "Cucuta",
    "CUE": "Cuenca",
    "CUK": "Caye Caulker",
    "CUL": "Culiacan",
    "CUM": "Cumana",
    "CUN": "Cancun",
    "CUP": "Carupano",
    "CUR": "Curacao",
    "CUS": "Custer, Sd",
    "CUT": "Cutral Co Ne",
    "CUU": "Chihuahua",
    "CUV": "Cuernavaca",
    "CUZ": "Cuzco",
    "CVG": "Cincinnati",
    "CVM": "Ciudad Victoria",
    "CVN": "Clovis",
    "CWA": "Wausau",
    "CWB": "Curitiba",
    "CWL": "Cardiff",
    "CWT": "Cowra",
    "CXJ": "Caxias Do Sul ",
    "CXL": "Al Bayda",
    "CXYL": "Cayo Largo",
    "CYB": "Cayman Brac Island",
    "CYR": "Colonia",
    "CZA": "Chichen Itza",
    "CZM": "Cozumel",
    "CZS": "Cruzeiro Do Sul",
    "CZU": "Corozal",
    "DAB": "Daytona Beach",
    "DAC": "Dhaka",
    "DAD": "Danang",
    "DAE": "Daparizo",
    "DAI": "Darjeeling",
    "DAK": "Dakhla Oasis",
    "DAK1": "White Desert ",
    "DAK2": "Farafra Oasis",
    "DAM": "Damascus",
    "DAM1": "Bosra",
    "DAM2": "Hama",
    "DAM3": "Krak des Chevaliers",
    "DAM4": "Homs",
    "DAM5": "Maaloula",
    "DAN": "Danbury",
    "DAR": "Dar es Salaam",
    "DAT": "Datong",
    "DAY": "Dayton, Ohio",
    "DBD": "Dhanbad",
    "DBO": "Dubbo",
    "DBP": "Dien Bein Phu",
    "DBQ": "Dubuque",
    "DDC": "Dodge City",
    "DEB": "Debre Zeyit",
    "DEC": "Decatur",
    "DED": "Dehradun",
    "DEL": "Delhi",
    "DEM": "Dembidollo",
    "DEN": "Denver",
    "DEO": "Deogarh",
    "DEO1": "Deosai Plains",
    "DER": "Del Rio",
    "DES": "M'Hamid",
    "DEZ": "Deir-Ez-Zor",
    "DFW": "Dallas",
    "DGA": "Dangriga",
    "DGO": "Durango",
    "DHA": "Dhahran",
    "DHL": "Dalhousie",
    "DHM": "Dharamsala- Mcleod Ganj",
    "DHN": "Dothan",
    "DHO": "Doha",
    "DIB": "Dibrugarh",
    "DID": "Dire Dawa",
    "DIJ": "Dijon",
    "DIN": "Dingle Bay",
    "DIP": "Dieppe Harbour",
    "DIY": "Diyarbakir",
    "DJA": "Dangriga",
    "DJG": "Djanet Inedbirene",
    "DKI": "Dunk Island",
    "DKR": "Dakar",
    "DLA": "Douala",
    "DLC": "Dalian",
    "DLH": "Duluth",
    "DLI": "Dalat",
    "DLM": "Dalaman",
    "DLU": "Dali",
    "DLZ": "Dalanzadgad",
    "DLZ1": "Bayanzag",
    "DLZ2": "Saikhan Ovoo",
    "DLZ3": "Hongoryn Els",
    "DLZ4": "South Gobi",
    "DLZ5": "Yolyn Am Canyon",
    "DLZ6": "The Mongolian Gobi",
    "DLZ7": "Lake Hovsgol",
    "DMR": "Damaraland",
    "DMU": "Dimapur",
    "DND": "Dundee",
    "DNH": "Dunhuang",
    "DNH1": "Liuyuan",
    "DNM": "Denham",
    "DNV": "Danville",
    "DOG": "Dogon Region",
    "DOH": "Doha",
    "DOK": "Donetsk",
    "DOM": "Dominica",
    "DOP": "Dolpo",
    "DOU": "Dourados",
    "DOV": "Dover",
    "DPL": "Dipolog",
    "DPO": "Devonport",
    "DPS": "Denpasar-Bali",
    "DRS": "Dresden",
    "DRW": "Darwin",
    "DSI": "Destin",
    "DSM": "Des Moines",
    "DTH": "Death Valley",
    "DTM": "Dortmund",
    "DTW": "Detroit",
    "DUB": "Dublin",
    "DUC": "Duck Key",
    "DUD": "Dunedin",
    "DUG": "Dungarpur",
    "DUI": "Danum Valley ",
    "DUJ": "Du Bois",
    "DUL": "Rio Dulce",
    "DUM": "Dumfries",
    "DUN": "Dun Laoghaire",
    "DUQ": "Duncan / Quam",
    "DUR": "Durban",
    "DUS": "Duesseldorf",
    "DUT": "Dutch Harbor",
    "DVO": "Davao",
    "DWN": "Darwin",
    "DWW": "Delaware Water Gap",
    "DXB": "Dubai",
    "DYU": "Dushanbe",
    "DYU1": "Istarafshan",
    "EAR": "Kearney",
    "EAS": "San Sebastian",
    "EAT": "Wenatchee",
    "EAU": "Eau Claire",
    "EBA": "Elba Island",
    "EBB": "Entebbe",
    "EBG": "El Bagre",
    "EBJ": "Esbjerg",
    "EC1": "The Cloud Forest",
    "EC2": "The Avenue of the volcanoes",
    "EC4": "Guamote",
    "EC5": "The Quilotoa Loop",
    "EDI": "Edinburgh",
    "EEN": "Keene / Brattleboro",
    "EGE": "Vail/Eagle",
    "EGL": "Erg Lihouidi ",
    "EHL": "El Bolson Rn",
    "EIN": "Eindhoven",
    "EIS": "Beef Island",
    "EKO": "Elko",
    "ELD": "El Dorado",
    "ELF": "El Fasher",
    "ELH": "North Eleuthera",
    "ELI": "Elim",
    "ELJ": "El Recreo",
    "ELL": "Ellsworth",
    "ELM": "Elmira / Corning",
    "ELP": "El Paso",
    "ELS": "East London",
    "ELV": "Elqui Valley",
    "ELY": "Ely Nv",
    "EMA": "East Midlands",
    "EMB": "Uda Walawe National Park",
    "EMD": "Emerald",
    "EMX": "El Maiten",
    "ENA": "Kenai",
    "ENC": "Pench Tiger Reserve",
    "ENN": "Ennis",
    "ENS": "Enschede",
    "ENT": "Ennistymon",
    "EOH": "M E Olaya Herrerra",
    "EPR": "Esperance",
    "EPT": "El Peten",
    "EQS": "Esquel ",
    "EQV": "Elqui Valley",
    "ERF": "Erfurt",
    "ERG": "Erg Chebbi",
    "ERI": "Erie",
    "ERS": "Windhoek",
    "ERV": "Kerriville",
    "ERZ": "Erzurum",
    "ESB": "Eastbourne",
    "ESC": "Escanaba",
    "ESD": "Eastsound",
    "ESE": "Ensenada",
    "ESF": "Alexandria",
    "ESM": "Esmeraldas",
    "ESN": "Ensenada",
    "ESQ": "Esquipulas",
    "ESR": "El Salvador",
    "EST": "Easton, Maryland",
    "EST1": "Esteros del Ibera Wetlands",
    "EST2": "Estancia",
    "ET1": "Harar",
    "ET4": "Nechisar National Park",
    "ETH": "Eilat",
    "ETSH": "Etosha National Park",
    "ETZ": "Metz/Nancy",
    "EUG": "Eugene",
    "EUN": "Laayoune",
    "EUR": "Eureka Ca",
    "EVE": "Evenes",
    "EVN": "Yerevan",
    "EVN0": "Dilijan",
    "EVN1": "Lake Sevan",
    "EVN2": "Sissian",
    "EVN3": "Dzoraget",
    "EVN4": "Gyumri",
    "EVN5": "Tsakhadzor",
    "EVN6": "Jermuk",
    "EVN7": "Bavra Border",
    "EVN8": "Amberd Fortress",
    "EVN9": "Echmiadzin",
    "EVV": "Evansville",
    "EWB": "New Bedford/Fall River",
    "EWN": "New Bern",
    "EWR": "Newark",
    "EXT": "Exeter",
    "EYP": "El Yopal",
    "EYW": "Key West",
    "EZE": "Ezezia International Airport",
    "F10": "Bokonbayevo",
    "FAE": "Faroe Islands",
    "FAI": "Fairbanks",
    "FAO": "Faro",
    "FAR": "Fargo",
    "FAT": "Fresno",
    "FAY": "Fayetteville",
    "FDF": "Fort De France",
    "FDH": "Friedrichshafen",
    "FEG": "Ferghana",
    "FEG1": "Dustlik Border",
    "FEN": "Fernando De Noronha",
    "FER": "Co. Fermanagh",
    "FEZ": "Fez",
    "FEZ2": "Asilah",
    "FEZ4": "Toubkal Refuge",
    "FFM": "Fergus Falls",
    "FHU": "Fort Huachuca/Sierra Vista",
    "FIH": "Kinshasa",
    "FIS": "Fishguard",
    "FIV": "Fivemiletown",
    "FKL": "Franklin",
    "FL": "Florida",
    "FLA": "Florencia",
    "FLG": "Flagstaff",
    "FLK": "Falkirk",
    "FLL": "Fort Lauderdale",
    "FLN": "Florianopolis",
    "FLO": "Florence",
    "FLR": "Florence",
    "FLS": "Flagstaff Az",
    "FMA": "Formosa",
    "FMB": "Fort Myers Beach",
    "FMN": "Farmington",
    "FMO": "Muenster",
    "FMY": "Fort Myers",
    "FNC": "Funchal",
    "FNG4": "Mount Myohyang",
    "FNG5": "Lake Sijung",
    "FNJ": "Pyongyang",
    "FNJ1": "Kaesong",
    "FNJ2": "Kumgang Mountains",
    "FNJ3": "Wonson",
    "FNL": "Fort Collins/Loveland",
    "FNT": "Flint",
    "FOD": "Fort Dodge",
    "FOE": "Topeka",
    "FOR": "Fortaleza ",
    "FPO": "Freeport",
    "FRA": "Frankfurt",
    "FRC": "Fish River Canyon",
    "FRD": "Fredericksburg Va",
    "FRM": "Fairmont",
    "FRO": "Floro",
    "FRS": "Flores",
    "FRU": "Bishkek",
    "FRU1": "Lake Issyk Kul",
    "FRU2": "Karakol",
    "FRU3": "Lake Issyk Kul (South Side)",
    "FRU4": "Koch Kor",
    "FRU5": "Naryn",
    "FRU6": "Jety Oguz Canyon",
    "FRU7": "Son Kul Lake",
    "FRU8": "Karkara Border",
    "FRU9": "Tash Rabat",
    "FSD": "Sioux Falls",
    "FSM": "Fort Smith",
    "FSP": "St Pierre",
    "FTE": "El Calafate",
    "FUE": "Fuerteventura / Puerto Del Ros",
    "FUK": "Fukuoka",
    "FWA": "Fort Wayne",
    "FYV": "Fayetteville",
    "GAI": "Gainseville",
    "GAJ": "Yamagata",
    "GAO": "Guantanamo Los Canos",
    "GAU": "Gauhati",
    "GAY": "Gaya",
    "GBE": "Gaborone",
    "GBG": "Galesburg",
    "GCC": "Gillette",
    "GCI": "Guernsey",
    "GCK": "Garden City",
    "GCM": "Grand Cayman",
    "GCN": "Grand Canyon",
    "GD": "Gondal",
    "GDL": "Guadalajara",
    "GDN": "Gdansk",
    "GDQ": "Gonder",
    "GDT": "Grand Turk Is",
    "GDV": "Glendive",
    "GDX": "Magadan",
    "GE1": "Mtskheta",
    "GE2": "Gori",
    "GE3": "Uplistsikhe",
    "GE4": "Vardzia",
    "GEL": "Santo Angelo ",
    "GEL1": "Gelephu",
    "GEO": "Georgetown",
    "GET": "Geraldton",
    "GFK": "Grand Forks",
    "GGG": "Longview/Gladewater/Kilgore",
    "GGS": "Gobernador Gregores",
    "GGT": "George Town",
    "GHB": "Governors Harbour",
    "GIB": "Gibraltar",
    "GIL": "Gilgit",
    "GIL2": "Hunza",
    "GIL3": "Besham",
    "GIL4": "Sost ",
    "GIO": "Giron",
    "GIR": "Girona",
    "GIU": "Sigiriya",
    "GIV": "Girvan",
    "GJA": "Guanaja",
    "GJK": "Gjiorkaster",
    "GKA": "Goroka",
    "GL": "Gloucester",
    "GLA": "Glasgow",
    "GLD": "Goodland",
    "GLF": "Golfito",
    "GLL": "Galle",
    "GLS": "Galveston",
    "GLV": "Guardalavaca",
    "GMC": "Guerima",
    "GNB": "Grenoble",
    "GND": "Grenada",
    "GNG": "Gorongosa National Park",
    "GNII": "The Ganges",
    "GNM": "Guanambi ",
    "GNP": "Glacier National Pk",
    "GNR": "Genaral Roca ",
    "GNV": "Gainesville",
    "GOA": "Genoa",
    "GOD": "Gondal",
    "GOH": "Nuuk",
    "GOI": "Goa",
    "GOJ": "Nizhniy Novgorod",
    "GON": "Gulf of Nicoya",
    "GOP": "Gorakhpur",
    "GOR": "Goris",
    "GPI": "Guapi",
    "GPR": "Gangapur",
    "GPS": "Galapagos",
    "GPT": "Gulfport",
    "GRB": "Green Bay",
    "GRF": "Great Falls Mt",
    "GRI": "Grand Island",
    "GRJ": "Grand Junction Co",
    "GRO": "Gerona",
    "GRP": "Gurupi",
    "GRQ": "Groningen",
    "GRR": "Grand Rapids",
    "GRZ": "Graz",
    "GSM": "Great Smoky Mountain",
    "GSO": "Greensboro",
    "GSP": "Greenville-Spartanbg",
    "GT1": "Lake Atitlan",
    "GTF": "Great Falls",
    "GTN": "Grand Teton Nat Park",
    "GTO": "Gorontalo",
    "GTY": "Gettysburg",
    "GUA": "Guatemala City",
    "GUB": "Guerrero Negro",
    "GUI": "Guiria",
    "GUJ": "Gujarat",
    "GUL": "Gulmarg",
    "GUM": "Guam",
    "GUN": "Guanajuato",
    "GUP": "Guapiles",
    "GUQ": "Guanare",
    "GUR": "Alotau",
    "GUS": "Gunnison",
    "GUU": "Gulmit",
    "GUX": "Guna",
    "GVA": "Geneva",
    "GVR": "Governador Valadares",
    "GWD": "Gwadar",
    "GWL": "Gwalior",
    "GWT": "Westerland",
    "GWY": "Galway",
    "GXF": "Seiyum",
    "GXQ": "Coyhaique",
    "GYE": "Guayaquil",
    "GYM": "Guaymas",
    "GYN": "Goiania",
    "GYY": "Gary",
    "GZW": "Qazvin",
    "HAA": "Havvnnes",
    "HAA1": "Haa Valley",
    "HAB": "Habarana",
    "HAD": "Halmstad",
    "HAG": "Ha Giang ",
    "HAH": "Moroni",
    "HAJ": "Hanover",
    "HAK": "Haikou",
    "HAKO": "Higashinada-ku",
    "HAM": "Hamburg",
    "HAN": "Hanoi",
    "HAN1": "Halong Bay",
    "HAN2": "Hoi An",
    "HAN3": "Lao Cai",
    "HAN4": "Sapa",
    "HAN5": "Giang Ta Chai",
    "HAN6": "Lang Son",
    "HAN7": "Cao Bang",
    "HAN8": "Ba Be Lake National Park",
    "HAN9": "Mekong Delta",
    "HAR": "Nothing",
    "HAT": "Hato Pinero",
    "HAU": "Haugesund",
    "HAV": "Havana",
    "HBA": "Hobart",
    "HBX": "Hubli",
    "HCJ": "Hito Cajon",
    "HDB": "Heidelberg",
    "HDK": "HD Kote Taluk",
    "HDN": "Hayden",
    "HDY": "Hat Yai",
    "HEH": "Heho",
    "HEL": "Helsinki",
    "HER": "Heraklion",
    "HET": "Hohhot",
    "HFT": "Hammerfest",
    "HGH": "Hangzhou",
    "HGN": "Mae Hong Son",
    "HGR": "Hagerstown",
    "HHH": "Hilton Head",
    "HHI": "Hilton Head Island",
    "HHQ": "Hua Hin ",
    "HIB": "Hibbing / Chisholm",
    "HIJ": "Hiroshima",
    "HIJ1": "Miyajima",
    "HIM": "Himeji",
    "HIR": "Honiara/Guadalcanal",
    "HIS": "Hayman Island",
    "HJR": "Khajuraho",
    "HJT": "Huatajata",
    "HKD": "Hakodate",
    "HKG": "Hong Kong",
    "HKK": "Hokitika",
    "HKN": "Hoskins",
    "HKT": "Phuket",
    "HKT1": "Koh Yao",
    "HKY": "Hickory",
    "HLB": "Holbrook Az",
    "HLN": "Helena",
    "HMA": "Malmo",
    "HMI": "Hami",
    "HML": "Hamilton",
    "HMO": "Hermosillo",
    "HMP": "Hampi",
    "HNA": "Morioka",
    "HNH": "Hoonah",
    "HNI": "Honarai",
    "HNL": "Honolulu",
    "HNS": "Haines",
    "HOB": "Hobbs",
    "HOD": "Hodeidah",
    "HOG": "Holguin",
    "HOL": "Holyhead",
    "HOM": "Homosassa",
    "HON": "Huron",
    "HOT": "Hot Springs",
    "HOU": "Houston",
    "HPB": "Hooper Bay",
    "HPH": "Haiphong",
    "HPN": "Westchester County",
    "HPV": "Kauai Island",
    "HRB": "Harbin",
    "HRD": "Haridwar",
    "HRE": "Harare",
    "HRG": "Hurghada",
    "HRK": "Kharkov",
    "HRL": "Harlingen",
    "HRO": "Harrison",
    "HSG": "Sagano Arashiyama",
    "HSI": "Hastings",
    "HSS": "Hissar",
    "HSV": "Huntsville/Decatur",
    "HTB": "Hato El Burro",
    "HTI": "Hamilton Island",
    "HTN": "Hotan",
    "HTS": "Huntington / Ashland",
    "HUF": "Terre Haute",
    "HUI": "Hue",
    "HUI1": "Bach Ma National Park",
    "HUL": "Hull",
    "HUN": "Hualien",
    "HUT": "Hutchinson Island",
    "HUU": "Huanuco",
    "HUX": "Huatulco",
    "HUY": "Humberside",
    "HVB": "Hervey Bay",
    "HVN": "New Haven",
    "HVR": "Havre",
    "HWD": "Holywood",
    "HYA": "Hyannis",
    "HYD": "Hyderabad",
    "HYG": "Hydaburg",
    "HYS": "Hays",
    "HZZ": "Huaraz",
    "I3N": "Marwar Region",
    "IAG": "Niagara Falls",
    "IAR": "Yaroslavl",
    "IAR1": "Rostov Veliky",
    "IAR2": "Pereslavl Zalessky",
    "IAS": "Iasi",
    "IBB": "Ibarra",
    "IBE": "Ibague",
    "IBZ": "Ibiza",
    "ICA": "Ica",
    "ICJ": "Parnaiba",
    "ICT": "Wichita",
    "IDA": "Idaho Falls",
    "IDM": "Ilha de Moambique",
    "IDR": "Indore",
    "IDS": "Isla del Sol",
    "IEV": "Kiev",
    "IFN": "Isfahan",
    "IGA": "Inagua",
    "IGM": "Kingman",
    "IGR": "Iguazu",
    "IGU": "Iguacu",
    "IIA": "Inishmaan",
    "IJK": "Izhevsk",
    "IKT": "Irkutsk",
    "IKT2": "Lake Baikal",
    "ILE": "Killeen",
    "ILF": "Ilford",
    "ILG": "Ilha Grande",
    "ILI": "Iliamna",
    "ILO": "Iloilo",
    "ILQ": "Ilo",
    "ILY": "Islay",
    "IMF": "Imphal",
    "IMP": "Imperatriz",
    "IMT": "Iron Mountain",
    "IN1": "Kalmatia",
    "IN11": "Kaziranga National Park",
    "IN12": "Nagaland",
    "IN14": "Assam",
    "IN16": "Varkala",
    "IN17": "Backwaters",
    "IN18": "Bijolia",
    "IN2": "Deora",
    "IN21": "Sorsan Wildlife Sanctuary",
    "IN25": "Kashmir",
    "IN26": "Ladakh",
    "IN27": "Ajanta Caves",
    "IN28": "Ellora Caves",
    "IN3": "Alai",
    "IN30": "Mamallapuram",
    "IN31": "Kalimpong",
    "IN32": "Marwar Junction",
    "IN33": "Sikkim",
    "IN35": "Fatehpur Sikri",
    "IN36": "Kumbhalgarh",
    "IN4": "Jawalbanj",
    "IN5": "Mukteshwar and Sitla Estate",
    "IN6": "Mararikulam",
    "IN7": "Corbett National Park",
    "IN9": "Leti",
    "INB": "Inhambane",
    "INC": "Yinchuan",
    "IND": "Indianapolis",
    "INL": "International Falls",
    "INN": "Innsbruck",
    "INQ": "Inisheer",
    "INSL": "Paliu",
    "INU": "Nauru",
    "INV": "Inverness",
    "IOA": "Ioannina",
    "IOM": "Isle Of Man",
    "IOO": "San Ignacio",
    "IOR": "Inishmore",
    "IOS": "Ilheus",
    "IPC": "Easter Island",
    "IPG": "Ipiranga",
    "IPH": "Ipoh",
    "IPI": "Ipiales",
    "IPN": "Ipatinga ",
    "IQQ": "Iquique",
    "IQT": "Iquitos",
    "IR01": "Co.Tipperary",
    "IR1": "Esfahan",
    "IR2": "The Caspian coast",
    "IRE": "Ireland",
    "IRJ": "La Rioja",
    "IRV": "Irvine",
    "ISB": "Islamabad",
    "ISC": "Isles Of Scilly",
    "ISG": "Ishigaki Island",
    "ISK": "Nasik Gandhinagr",
    "ISL": "Islamorada",
    "ISM": "Isla Mujeres",
    "ISN": "Williston",
    "ISO": "Kinston",
    "ISP": "Islip",
    "IST": "Istanbul",
    "IST1": "Canakkale",
    "IST2": "Dogubayazit",
    "IST3": "Kahta",
    "IST4": "Ephesus",
    "IST8": "Troy",
    "ITC": "Itacare",
    "ITH": "Ithaca",
    "ITO": "Hilo",
    "ITP": "Itaparica",
    "ITV": "Itaipava",
    "IVC": "Invercargill",
    "IWD": "Ironwood",
    "IXA": "Agartala Singerbhil",
    "IXB": "Bagdogra",
    "IXC": "Chandigarh",
    "IXD": "Allahabad",
    "IXE": "Mangalore",
    "IXG": "Belgaum Sambre",
    "IXH": "Kailashahar",
    "IXI": "Lilabari",
    "IXJ": "Jammu",
    "IXK": "Keshod",
    "IXL": "Leh",
    "IXM": "Madurai",
    "IXM2": "Kodaikanal",
    "IXN": "Khowai",
    "IXP": "Pathankot",
    "IXQ": "Kamalpur",
    "IXR": "Ranchi",
    "IXS": "Silchar Kumbirgram",
    "IXT": "Ixtapa",
    "IXU": "Aurangabad",
    "IXV": "Along",
    "IXW": "Jamshedpur Sonari",
    "IXY": "Kandla",
    "IXZ": "Port Blair",
    "IYK": "Inyokern",
    "IZO": "Izumo",
    "JAC": "Jackson Hole",
    "JAF": "Kanker",
    "JAI": "Jaipur",
    "JAK": "Jackson Ms",
    "JAN": "Jackson",
    "JAR": "Jarar",
    "JAT": "Jabat",
    "JAV": "Ilulissat",
    "JAX": "Jacksonville",
    "JBR": "Jonesboro",
    "JCA": "Cannes",
    "JCO": "Jaco",
    "JDH": "Jodhpur",
    "JDO": "Juazeiro Do Norte ",
    "JED": "Jeddah",
    "JEN": "Jensen Beach Fl",
    "JER": "Jersey",
    "JGA": "Jamnagar",
    "JGB": "Jagdalpur",
    "JGN": "Jiayuguan",
    "JGU": "Jaguanum Island",
    "JHA": "Jhalawar",
    "JHB": "Johor Bahru",
    "JHE": "Helsingborg",
    "JHG": "Jinghong",
    "JHG1": "Mohan/Boten Border",
    "JHM": "Kapalua",
    "JHN": "Jhansi",
    "JHW": "Jamestown",
    "JIB": "Djibouti",
    "JIN": "Junin De Los Andes",
    "JJI": "Juanjui",
    "JKG": "Jonkoping",
    "JKT": "Jakarta",
    "JLN": "Joplin",
    "JLR": "Jabalpur",
    "JMC": "Sausalito",
    "JMK": "Mikonos",
    "JMM": "Malmo",
    "JMO": "Jomsom",
    "JMS": "Jamestown",
    "JNB": "Johannesburg",
    "JNS": "Narssaq",
    "JNU": "Juneau",
    "JNX": "Naxos",
    "JO1": "The KingÕs Highway",
    "JO2": "MaÕIn",
    "JOI": "Joinville ",
    "JOJ": "Jojowar",
    "JOL": "La Jolla",
    "JON": "Johnston Island",
    "JPA": "Joao Pessoa",
    "JPR": "Jiparana Ro",
    "JRA": "New York City",
    "JRC": "Jericoacoara",
    "JRH": "Jorhat Rowriah",
    "JRO": "Kilimanjaro",
    "JSA": "Jaisalmer",
    "JSI": "Skiathos",
    "JSM": "Jose De San Martin",
    "JST": "Johnstown",
    "JTR": "Santorini/Thira Is",
    "JUG": "Gallon Jug",
    "JUJ": "Jujuy ",
    "JUL": "Juliaca",
    "JZH": "Jiuzhaigou",
    "KAB": "Kariba",
    "KAE": "Kake",
    "KAK": "Kalai-Khumb",
    "KAL": "Kalispell",
    "KAL1": "The Kalash",
    "KALA": "Kalaw",
    "KALL": "Kalpitya",
    "KAN": "Kanab",
    "KANA": "Kanan",
    "KAR": "Kazarman",
    "KAR1": "Karaikudi ",
    "KAS": "Kasaragod",
    "KAT": "Kaitaia",
    "KAV": "Kavanayen",
    "KAW": "Kawardha",
    "KBI": "Chi Phat ",
    "KBL": "Kabul",
    "KBP": "Kiev",
    "KBR": "Kota Bharu",
    "KBV": "Krabi",
    "KCA": "Kuqa",
    "KCG": "Chignik",
    "KCH": "Kuching",
    "KCH1": "Bako National Park",
    "KCH2": "Semenggoh Nature Reserve",
    "KCH3": "Batang Ai",
    "KCH4": "Nanga Sumpa",
    "KCL": "Chignik",
    "KDU": "Skardu",
    "KDW": "Kandy",
    "KE1": "Samburu",
    "KE2": "Northern Frontier",
    "KE3": "Lake Naivasha and Lake Nakuru",
    "KE5": "Masai Mara",
    "KE6": "Laikipia",
    "KE7": "Coastal Kenya",
    "KE8": "Meru",
    "KE9": "Amboseli ",
    "KEH": "Kenmore Air Harbor",
    "KEJ": "Kemerovo",
    "KEL": "Kiel",
    "KEL2": "Kelani River",
    "KEM": "Kemi/Tornio",
    "KEN": "Kennebunkport, Me",
    "KEP": "Nepalganj",
    "KER": "Kerman",
    "KET": "Keng Tung",
    "KEY": "Keystone",
    "KGC": "Kingscote",
    "KGD": "Kaliningrad",
    "KH1": "Kep",
    "KH2": "The Cardamom Mountains Region",
    "KH3": "Tatai ",
    "KHA": "Kanha National Park",
    "KHD": "Central Kalahari Game Reserve",
    "KHG": "Kashgar",
    "KHG1": "Yarkand",
    "KHG2": "Minfeng",
    "KHG3": "Torugart Pass",
    "KHG4": "Tashkurgan",
    "KHG5": "Irkestan Pass",
    "KHH": "Kaohsiung",
    "KHI": "Karachi",
    "KHM": "Khimsar",
    "KHV": "Khabarovsk",
    "KHW": "Khandwa",
    "KI": "Keek",
    "KID": "Co. Kildare",
    "KII": "Kilpisjarvi",
    "KIK": "Kilkee",
    "KIL": "Killington",
    "KIN": "Kingston",
    "KIO": "Killaloe",
    "KIR": "Kerry",
    "KIV": "Kishinev",
    "KJA": "Krasnojarsk",
    "KKN": "Kirkenes",
    "KKV": "Kavac",
    "KKY": "Kilkenny",
    "KLH": "Kolhapur",
    "KLI": "Karauli",
    "KLK": "Kilkeel",
    "KLN": "Killarney",
    "KLO": "Kalibo",
    "KLR": "Kalmar",
    "KLU": "Klagenfurt",
    "KLW": "Klawock",
    "KMG": "Kunming",
    "KMG1": "Jianshui",
    "KMG2": "Yuanyang",
    "KMI": "Miyazaki",
    "KMJ": "Kumamoto",
    "KMK": "Kilmarnock",
    "KMT": "Kampot",
    "KNA": "Vina Del Mar",
    "KNP": "Kafue National Park",
    "KNPP": "Katavi National Park",
    "KNS": "King Island",
    "KNU": "Kanpur",
    "KNX": "Knoxville, Tn",
    "KOA": "Kona",
    "KOI": "Kirkwall",
    "KOJ": "Kagoshima",
    "KOK": "Kokkola/Pietarsaari",
    "KOR": "Playa Koralia",
    "KOS": "Sihanoukville",
    "KOT": "Kottayam",
    "KOV": "Kovalam",
    "KOY": "Koyten",
    "KPA": "Keladan Pengirup Ai",
    "KPO": "Pohang",
    "KRG": "Kargil",
    "KRK": "KRAKOW",
    "KRL": "Korla",
    "KRN": "Kiruna",
    "KRR": "Krasnodar",
    "KRS": "Kristiansand",
    "KRT": "Khartoum",
    "KRW": "Turkmenbashi",
    "KSA": "Kosrae",
    "KSC": "Kosice",
    "KSH": "Kermanshah",
    "KSH1": "Hamedan",
    "KSJ": "Kasos Island",
    "KSU": "XXXX",
    "KSY": "Kars",
    "KT2": "Koh Trong",
    "KTG": "Kathgodam",
    "KTI": "Kratie",
    "KTK": "Karnataka",
    "KTM": "Kathmandu",
    "KTN": "Ketchikan",
    "KTR": "Katherine",
    "KTU": "Kota",
    "KTV": "Kamarata",
    "KTW": "Katowice",
    "KUA": "Kuantan",
    "KUD": "Kuldhara",
    "KUF": "Samara",
    "KUL": "Kuala Lumpur",
    "KUN": "Kaunas",
    "KUN1": "Khunjarab Pass",
    "KUO": "Kuopio",
    "KUS": "Kulusuk",
    "KUT": "Kutaisi",
    "KUU": "Kulu Bhuntar",
    "KUV": "Kunsan",
    "KVA": "Kavala",
    "KVD": "Ganja",
    "KWA": "Kwajalein",
    "KWE": "Guiyang",
    "KWE1": "Kaili",
    "KWE2": "Rongjiang",
    "KWE3": "Zhaoxing",
    "KWI": "Kuwait City",
    "KWJ": "Gyeongju",
    "KWL": "Guilin",
    "KWL1": "Yangshuo",
    "KWL2": "Sanjiang",
    "KWL3": "Longsheng ",
    "KYE": "Tripoli",
    "KYE1": "Abboudieh/Dabussieh Border",
    "KYL": "Key Largo",
    "KZ1": "Tien Shan Mountains",
    "KZK": "Kompong Thom",
    "KZN": "Kazan",
    "LAC": "Las Cruces Nm",
    "LAD": "Luanda",
    "LAF": "Lafayette",
    "LAG": "Lagos",
    "LAH": "Lahinch",
    "LAJ": "Lages ",
    "LAK": "Lake City",
    "LAN": "Lancaster Pa",
    "LAO": "Co. Laois",
    "LAP": "La Paz ( Mexico)",
    "LAQ": "La Quiaca",
    "LAR": "Laramie",
    "LAS": "Las Vegas",
    "LAT": "Latacunga/Cotopaxi",
    "LAU": "Laughlin",
    "LAW": "Lawton",
    "LAX": "Los Angeles",
    "LAZ": "Bom Jesus Da Lapa ",
    "LBA": "Leeds",
    "LBB": "Lubbock",
    "LBD": "Khojand",
    "LBE": "Latrobe",
    "LBF": "North Platte",
    "LBK": "Longboat Key Fl",
    "LBL": "Liberal",
    "LBU": "Labuan",
    "LBV": "Libreville",
    "LCA": "Larnaca",
    "LCD": "La Candelaria",
    "LCE": "La Ceiba",
    "LCG": "La Coruna",
    "LCH": "Lake Charles",
    "LCR": "La Correra",
    "LDA": "Malda",
    "LDB": "Londrina ",
    "LDE": "Co. Londonderry",
    "LDU": "Lahad Datu",
    "LDV": "Lisdoonvarna",
    "LDY": "Londonderry",
    "LEA": "Learmonth",
    "LEB": "Lebanon/Hanover/White River",
    "LEC": "Chapada Diamantina",
    "LED": "St Petersburg",
    "LEH": "Le Havre",
    "LEI": "Co. Leitrim",
    "LEJ": "Leipzig",
    "LEN": "Lencois",
    "LEQ": "Lands End",
    "LET": "Leticia",
    "LEX": "Lexington",
    "LFT": "La Fortuna",
    "LFW": "Lome",
    "LGB": "Long Beach",
    "LGG": "Liege",
    "LGH": "Loughborough",
    "LGI": "Deadmans Cay / Long Island",
    "LGP": "Legaspi",
    "LGQ": "Lagoagrio",
    "LGZ": "Leguizamo",
    "LHE": "Lahore",
    "LHV": "Lake Havasu Az",
    "LHW": "Lanzhou",
    "LHW1": "Xiahe",
    "LIG": "Limoges",
    "LIH": "Kauai Island",
    "LIL": "Lake Inle",
    "LIL1": "Taunggyi ",
    "LIM": "Lima",
    "LIO": "Limon",
    "LIR": "Liberia",
    "LIS": "Lisbon",
    "LIT": "Little Rock",
    "LIV": "Livingstone",
    "LJG": "Lijiang",
    "LJG1": "Zhongdian",
    "LJG2": "RINGHA",
    "LJU": "Ljubljana",
    "LK1": "Aukana",
    "LK2": "The Knuckles Mountain Range",
    "LKN": "Leknes",
    "LKO": "Lucknow",
    "LKY4": "Manasarovar Lake",
    "LKY7": "Yam drot so",
    "LKY8": "Ali",
    "LL1": "Lake District",
    "LLA": "Lulea",
    "LLG": "Los Linges",
    "LLI": "Lalibela",
    "LLW": "Lilongwe",
    "LLY": "Mount Holly",
    "LMC": "La Macarena",
    "LMK": "Limerick",
    "LMM": "Los Mochis",
    "LMN": "Limbang",
    "LMNP": "Lake Manyara National Park",
    "LMT": "Klamath Falls",
    "LNG": "Langar",
    "LNK": "Lincoln",
    "LNS": "LLanos",
    "LNV": "Londolovit",
    "LNY": "Lanai City",
    "LNZ": "Linz",
    "LOF": "The Lofoten Islands",
    "LOG": "Co. Longford",
    "LOH": "Loja",
    "LOM": "Lagos De Moreno",
    "LON": "London",
    "LOS": "Lagos",
    "LOU": "Louisville Ky",
    "LOV": "Monclover",
    "LPA": "Las Palmas",
    "LPB": "La Paz",
    "LPC": "Lipska Pecina Cave ",
    "LPD": "La Pedrera",
    "LPE": "La Primavera",
    "LPI": "Linkoping",
    "LPL": "Liverpool",
    "LPO": "Lago Posadas",
    "LPQ": "Luang Prabang",
    "LPQ1": "Kamu Lodge",
    "LPQ2": "Huay Xai",
    "LRD": "Laredo",
    "LRH": "La Rochelle",
    "LRM": "Casa De Campo",
    "LRS": "Los Roques",
    "LRT": "Lorient",
    "LRU": "Las Cruces",
    "LSC": "La Serena",
    "LSE": "La Crosse",
    "LSI": "Sumburgh, Shetlands",
    "LSP": "Las Piedras",
    "LSQ": "Los Angeles",
    "LSS": "Terre-De-Haut",
    "LST": "Launceston",
    "LSY": "Lismore",
    "LTBO": "Lake Titicaca Bolivia",
    "LTH": "Lake Tahoe Ca And Nv",
    "LTK": "Latakia and the Coast",
    "LTK1": "Tartus",
    "LTM": "Las Terrazas Moka",
    "LTO": "Loreto",
    "LTPE": "Lake Titicaca Peru",
    "LTX": "Cotopaxi National Park",
    "LUA": "Lukla",
    "LUD": "Luderitz",
    "LUDH": "Ludharva",
    "LUG": "Lugano",
    "LUH": "Ludhiana Sahnewal",
    "LUN": "Lusaka",
    "LUQ": "San Luis Sl",
    "LUT": "Co.Louth",
    "LUX": "Luxembourg",
    "LVV": "Livingston",
    "LWB": "Greenbrier",
    "LWK": "Lerwick, Shetlands",
    "LWL": "Linyanti Wetlands and Savuti",
    "LWN": "Geghard",
    "LWN1": "Garni",
    "LWN2": "Khor Virab",
    "LWN3": "Southern Armenia",
    "LWN4": "Tatev",
    "LWO": "Lvov",
    "LWS": "Lewiston",
    "LWT": "Lewistown",
    "LWY": "Lawas",
    "LXA": "Lhasa",
    "LXA1": "Shigatse",
    "LXA2": "Gyantse",
    "LXG": "Luang Nam Tha",
    "LXG1": "Muang Sin",
    "LXG2": "Ban Lao Khao",
    "LXG3": "Ban Tamee",
    "LXG4": "Muang La",
    "LXR": "Luxor",
    "LXR1": "El Kharga Oasis",
    "LXR2": "The Nile Valley",
    "LY1": "Ubari Sand Sea",
    "LY2": "Sabratha",
    "LYA": "Luoyang",
    "LYH": "Lynchburg",
    "LYM": "Newcastle-Under-Lyme",
    "LYR": "Longyearbyen",
    "LYS": "Lyon",
    "LZC": "Lazaro Cardenas",
    "LZNP": "Lower Zambezi National Park",
    "LZY1": "Gyantse",
    "LZY2": "Xigatse",
    "LZY3": "Everest Base Camp",
    "LZY5": "Mount Kailash",
    "LZY6": "The Kingdom of Guge",
    "MA1": "The Atlas Mountains",
    "MAA": "Chennai",
    "MAA1": "Rameswaram",
    "MAA2": "Tanjore",
    "MAA3": "Tranquebar",
    "MAA4": "Chidambaram",
    "MAA5": "Kanchipuram",
    "MAA6": "Chettinad",
    "MAB": "Maraba ",
    "MAC": "Mai Chau",
    "MAD": "Madrid",
    "MAF": "Midland/Odessa",
    "MAG": "Madang",
    "MAH": "Menorca",
    "MAI": "Manali",
    "MAJ": "Majuro",
    "MAL": "Masouleh",
    "MAM": "Matamoros",
    "MAN": "Manchester",
    "MAO": "Manaus",
    "MAR": "Maracaibo",
    "MAS": "Manassas, Va",
    "MAU": "Maraœ",
    "MAV": "Manwar",
    "MAW": "Mandawa",
    "MAY": "Co.Mayo",
    "MAZ": "Mayaguez",
    "MB": "Monte Alban",
    "MBJ": "Montego Bay",
    "MBK": " Mrauk U",
    "MBL": "Manistee",
    "MBS": "Midland / Bay City / Saginaw",
    "MBU": "Mount Abu",
    "MCE": "Merced",
    "MCH": "Machala",
    "MCI": "Kansas",
    "MCK": "Mc Cook",
    "MCM": "Monte Carlo",
    "MCN": "Macon",
    "MCP": "Macapa ",
    "MCT": "Muscat",
    "MCT1": "Barka",
    "MCT2": "Nizwa",
    "MCT3": "Jabel Akhdar",
    "MCT4": "Jabel Shams",
    "MCW": "Mason City",
    "MCX": "Mocoa",
    "MCY": "Sunshine Coast",
    "MCZ": "Maceio",
    "MDC": "Manado",
    "MDD": "Madaba",
    "MDE": "Medellin",
    "MDH": "Carbondale",
    "MDI": "Madidi Rainforest Reserve",
    "MDL": "Masnaa Border",
    "MDQ": "Mar Del Plata",
    "MDR": "Marina Del Rey",
    "MDT": "Harrisburg",
    "MDU": "Mandu",
    "MDZ": "Mendoza",
    "MEA": "Co.Meath",
    "MEC": "Manta",
    "MED": "Medinah",
    "MEI": "Meridian",
    "MEL": "Melbourne",
    "MEM": "Memphis",
    "MEN": "Mendocino Ca",
    "MER": "Meroe",
    "MES": "Medan",
    "MEX": "Mexico City",
    "MEY": "Meghauli",
    "MFE": "Mc Allen/Mission",
    "MFI": "Mafia Island",
    "MFM": "Macau",
    "MFN": "Milford Sound",
    "MFQ": "Maradi",
    "MFR": "Medford",
    "MFW": "Mfuwe",
    "MGA": "Managua",
    "MGF": "Maringa",
    "MGN": "Co.Monaghan",
    "MGQ": "Mogadishu",
    "MGR": "Moremi Game Reserve",
    "MGW": "Morgantown",
    "MHD": "Mashad",
    "MHD1": "Badjgiran Border",
    "MHH": "Marsh Harbour",
    "MHK": "Manhattan",
    "MHQ": "Mariehamn",
    "MHS": "Maheshwar",
    "MIA": "Miami",
    "MID": "Merida",
    "MIE": "Muncie",
    "MII": "Marilia",
    "MIL": "Milan",
    "MIR": "Monastir",
    "MIS": "Missoula Mt",
    "MJT": "Mytilene",
    "MJV": "Murcia",
    "MKC": "Kansas City",
    "MKE": "Milwaukee",
    "MKG": "Muskegon",
    "MKGD": "Makgadikgadi Pans",
    "MKK": "Hoolehua",
    "MKL": "Jackson",
    "MKM": "Mukah",
    "MKW": "Milwaukee",
    "MKX": "Mukalla",
    "MKY": "Mackay",
    "ML1": "Djenne",
    "MLA": "Malta",
    "MLC": "Mainland Coast",
    "MLE": "Male",
    "MLE1": "North Male Atoll",
    "MLE2": "South Male Atoll",
    "MLH": "Mulhouse",
    "MLI": "Moline",
    "MLM": "Morelia",
    "MLO": "Milos",
    "MLS": "Miles City",
    "MLT": "Milton Keynes",
    "MLU": "Monroe",
    "MMB": "Memanbetsu",
    "MMD": "Manmad",
    "MME": "Teeside",
    "MMJ": "Matsumoto",
    "MMK": "Murmansk",
    "MML": "Mammoth Lakes",
    "MMNP": "Mahale Mountains National park",
    "MMP": "Mompos",
    "MMU": "Morristown",
    "MMX": "Malmo",
    "MNI": "Montserrat",
    "MNL": "Manila",
    "MNM": "Menominee",
    "MNR": "Munnar",
    "MNR2": "Thekkady",
    "MNR3": "Kumarakom",
    "MNR4": "Trivandrum",
    "MNR5": "Neeleswaram",
    "MNR6": "Coonoor",
    "MNR7": "Coorg",
    "MNY": "Mandalay",
    "MO1": "Mondulkiri",
    "MOA": "Moa",
    "MOB": "Mobile",
    "MOD": "Modesto",
    "MOH": "Mohanbari",
    "MOJ": "Mojkovac ",
    "MOL": "Molde",
    "MON": "Montgomery",
    "MON1": "Mongar",
    "MOO": "Manuel Antonio",
    "MOS": "Molinos",
    "MOT": "Minot",
    "MOW": "Moscow",
    "MOW1": "Vladimir",
    "MOW2": "Suzdal",
    "MOY": "Morrocoy",
    "MPA": "Mpacha",
    "MPI": "Machu Picchu",
    "MPL": "Montpellier",
    "MPM": "Maputo",
    "MPN": "Mount Pleasant",
    "MPR": "Mountain Pine Ridge",
    "MQH": "Minacu",
    "MQL": "Mildura",
    "MQT": "Marquette",
    "MRA": "Marathon",
    "MRD": "Merida",
    "MRN": "Mt Rainier Nat Park",
    "MRR": "Morro de Sao Paulo",
    "MRS": "Marseille",
    "MRU": "Mauritius",
    "MRY": "Monterey",
    "MSJ": "Misawa",
    "MSL": "Muscle Shoals / Florence / She",
    "MSN": "Madison",
    "MSO": "Missoula",
    "MSP": "Minneapolis",
    "MSQ": "Minsk",
    "MSS": "Massena",
    "MST": "Maastricht",
    "MSU": "Maseru",
    "MSY": "New Orleans",
    "MTJ": "Montrose",
    "MTP": "Monument Valley Trib",
    "MTR": "Monteria",
    "MTT": "Minatitlan",
    "MTY": "Monterrey",
    "MUB": "Maun",
    "MUC": "Munich",
    "MUE": "Mauritius East Coast",
    "MUN": "Maturin",
    "MUO": "Mauritius North Coast",
    "MUP": "Port Louis",
    "MUR": "Murgab",
    "MUU": "Manu",
    "MUW": "Mauritius West Coast",
    "MVD": "Montevideo",
    "MVN": "Mesa Verde Nat Park",
    "MVP": "Mitu",
    "MVV": "Monteverde",
    "MVY": "Martha's Vineyard",
    "MWA": "Marion",
    "MXL": "Mexicali",
    "MXW": "Mandalgovi",
    "MXW1": "Bagarin Chuluu (Middle Gobi)",
    "MYA": "Moruya",
    "MYC": "Maracay",
    "MYJ": "Matsuyama",
    "MYO": "Maymyo",
    "MYP": "Merv",
    "MYP1": "Mary",
    "MYQ": "Mysore",
    "MYR": "Myrtle Beach",
    "MYY": "Miri",
    "MYY1": "Niah Caves",
    "MZ1": "Pemba",
    "MZ2": "Quirimbas Archipelago",
    "MZA": "Muzaffarnagar",
    "MZI": "Mopti",
    "MZL": "Manizales",
    "MZO": "Manzanillo",
    "MZR": "Mazar-i-Sharif",
    "MZT": "Mazatlan",
    "MZU": "Muzaffarpur",
    "MZV": "Mulu",
    "NAG": "Nagpur",
    "NAG1": "Nagarhole",
    "NAG3": "Nagarhole",
    "NAH": "Naha",
    "NAJ": "Nakhchivan",
    "NAN": "Nadi",
    "NAP": "Naples",
    "NAR": "Narlai",
    "NAS": "Nassau",
    "NAT": "Natal",
    "NAV": "Napa Valley Ca",
    "NAZ": "Natchez",
    "NBB": "Barrancominas",
    "NBO": "Nairobi",
    "NBV": "Nubra Valley",
    "NCA": "North Caicos",
    "NCE": "Nice",
    "NCI": "Necocli",
    "NCL": "Newcastle",
    "NCU": "Nukus",
    "NCY": "North Conway Nh",
    "NDC": "Nanded",
    "NDJ": "N Djamena",
    "NDS": "Namib Desert & Sossusvlei",
    "NE1": "Bilma",
    "NE2": "The A•r Massif",
    "NE3": "Djado",
    "NE4": "The TŽnŽrŽ Desert",
    "NEC": "Necochea",
    "NEE": "Neemrana",
    "NEL": "Neeleshwar",
    "NEP": "Newport Harbour",
    "NEV": "Nevis",
    "NEW": "Newport Or",
    "NFK": "Norfolk Va",
    "NGC": "Ngorongoro Crater",
    "NGO": "Nagoya",
    "NGS": "Nagasaki",
    "NHA": "Nha Trang",
    "NIA": "Niagara Falls Ny",
    "NIL": "The Nile",
    "NIM": "Nimaj",
    "NJC": "Nizhnevartovsk",
    "NKC": "Nouakchott",
    "NKG": "Nanjing",
    "NKP": "Noel Kempff National Park",
    "NLA": "Ndola",
    "NLD": "Nuevo Laredo",
    "NMB": "Daman",
    "NMG": "San Miguel",
    "NMI": "Naples/Marco Island",
    "NMP": "Mulu National Park",
    "NNG": "Nanning",
    "NOC": "Connaught",
    "NOH": "Nohur",
    "NOR": "Norwich",
    "NOT": "Nottingham",
    "NOU": "Noumea",
    "NPB": "Newport Beach Ca",
    "NPL": "New Plymouth",
    "NPN": "Newport News",
    "NPT": "Newport",
    "NQL": "Niquelanddia",
    "NQN": "Neuquen",
    "NQU": "Nuqui",
    "NQY": "Newquay",
    "NSB": "New Smyrna Beach",
    "NSI": "Yaounde",
    "NSN": "Nelson",
    "NST": "Nakhon Ratchasima",
    "NST1": "Khao Yai National Park",
    "NTE": "Nantes",
    "NTH": "Northwich",
    "NUA": "Nuwara Eliya",
    "NUA1": "Ella",
    "NUE": "Nuremberg",
    "NVA": "Neiva",
    "NVI": "Navoi",
    "NVI1": "Yangikasgan (Kyzyk-Kul Desert)",
    "NVT": "Navegantes",
    "NVY": "Neyveli",
    "NWA": "Moheli",
    "NWI": "Norwich",
    "NWP": "Newport Ri",
    "NWS": "Newtonstewart",
    "NYC": "New York",
    "NZA": "Nazca",
    "OAG": "Orange",
    "OAK": "Oakland",
    "OAX": "Oaxaca",
    "OBB": "Cheju",
    "OBO": "Obihiro",
    "OCC": "Coca",
    "OCV": "Ocana",
    "OCY": "Ocean City Md",
    "ODE": "Odense",
    "ODS": "Odessa",
    "ODW": "Oak Harbor",
    "ODY": "Oudomxay",
    "OES": "San Antonio Oeste Rn",
    "OFK": "Norfolk",
    "OGG": "Kahului",
    "OGS": "Ogdensburg",
    "OHD": "Ohrid",
    "OIT": "Oita",
    "OJO": "Ojos de Perdiz",
    "OK": "Okaloma City",
    "OKA": "Okinawa",
    "OKC": "Oklahoma City",
    "OKD": "Okavango Delta",
    "OKJ": "Okayama",
    "OLF": "Wolf Point",
    "OLI": "Olinda",
    "OLL": "Ollantaytambo",
    "OLY": "Olympia Washington",
    "OM1": "Hajar Mountains",
    "OMA": "Omaha",
    "OME": "Nome",
    "OMN": "Osmanabad",
    "OMR": "Oradea",
    "ONC": "Concepcion",
    "ONG": "Mornington",
    "ONT": "Ontario",
    "OOL": "Gold Coast",
    "OOM": "Cooma",
    "OOO": "Otavalo",
    "OOT": "Ooty",
    "OPO": "Oporto",
    "ORB": "Orebro",
    "ORC": "Orchha",
    "ORF": "Norfolk",
    "ORG": "Orange Ca",
    "ORH": "Worcester",
    "ORK": "Cork",
    "ORL": "Orlando",
    "ORN": "Oran",
    "ORU": "Oruro",
    "OSA": "Osaka",
    "OSD": "Ostersund",
    "OSH": "Oshkosh",
    "OSL": "Oslo",
    "OSS": "Osh",
    "OSS1": "Sary Tash",
    "OSS2": "Arslanbob Village",
    "OSS3": "Chychkan Gorge",
    "OSS4": "Ala Acha Gorge",
    "OSS6": "Samsy Valley National Park",
    "OSS7": "Tamga",
    "OSS8": "Mountains of Heaven",
    "OTA": "Otavalo",
    "OTH": "North Bend",
    "OTM": "Ottumwa",
    "OTZ": "Kotzebue",
    "OUA": "Ouagadougou",
    "OUC": "Ouen Casa",
    "OUL": "Oulu",
    "OUP": "Ouro Preto",
    "OUR": "Ourika",
    "OVB": "Novosibirsk",
    "OVD": "Oviedo",
    "OWB": "Owensboro",
    "OWD": "Norwood",
    "OWK": "Orange Walk",
    "OXB": "Bissau",
    "OXN": "Oxnard",
    "OXR": "Oxnard / Ventura",
    "OYS": "Yosemite",
    "OZZ": "Ouarzazate",
    "PAB": "Bilaspur",
    "PAD": "Paderborn",
    "PAG": "Pacific Grove Ca",
    "PAH": "Paducah",
    "PAI": "Paipa",
    "PAJ": "Panajachel",
    "PAK": "Pakbeng",
    "PAL": "Palitana",
    "PAM": "Pamukkale",
    "PANT": "Pantanal",
    "PAP": "Port Au Prince",
    "PAQ": "Pisaq",
    "PAR": "Paris",
    "PAS": "Paros",
    "PAT": "Patna",
    "PAU": "St.Paul Minnesota",
    "PAZ": "Poza Rica",
    "PBA": "Portballintrae",
    "PBC": "Puebla",
    "PBD": "Porbandar",
    "PBH": "Paro",
    "PBI": "Palm Beach",
    "PBM": "Paramaribo",
    "PBO": "Paraburdoo",
    "PCB": "Panama City Beach",
    "PCH": "Puerto Chacabuco",
    "PCL": "Pucallpa",
    "PCM": "Playa Del Carmen",
    "PCQ": "Phonsavan",
    "PCS": "Pacific Shore",
    "PDF": "Praia Do Forte",
    "PDG": "Porto de Galinhas",
    "PDL": "Ponta Delgada",
    "PDO": "Peninsula de Osa",
    "PDP": "Punta Del Este",
    "PDR": "Pinar del Rio",
    "PDT": "Pendleton",
    "PED": "San Pedro de Atacama",
    "PEE": "Perm",
    "PEG": "Perugia",
    "PEI": "Pereira",
    "PEK": "Beijing",
    "PEK1": "Chengde",
    "PEK3": "Hainan Island",
    "PEK4": "Taklamakan Desert",
    "PEK5": "Yangtze River",
    "PEM": "Puerto Maldonado",
    "PEMB": "Pemba Island",
    "PEN": "Penrith",
    "PER": "Perth",
    "PES": "Petrozavodsk",
    "PEW": "Peshawar",
    "PFB": "Passo Fundo",
    "PFN": "Panama City",
    "PFO": "Paphos",
    "PFR": "Puerto Frias",
    "PGE": "Page",
    "PGF": "Perpignan",
    "PGH": "Pantnagar",
    "PGL": "Puerto Guadal",
    "PGV": "Greenville",
    "PGX": "Perigueux",
    "PHA": "Phander",
    "PHE": "Port Hedland",
    "PHF": "Williamsburg",
    "PHH": "PHAN THIET",
    "PHL": "Philadelphia",
    "PHO": "Point Hope",
    "PHO1": "Phobjika",
    "PHX": "Phoenix",
    "PHY": "Puyuhuapi",
    "PI1": "Pipariya ",
    "PIA": "Peoria",
    "PIB": "Punta Islita",
    "PIC": "Picinguaba",
    "PIE": "Tampa/St.Petes",
    "PIH": "Pocatello",
    "PII": "Puri",
    "PIK": "Prestwick",
    "PIN1": "Palakkad",
    "PIO": "Pisco",
    "PIP": "Pipa",
    "PIR": "Pierre",
    "PIT": "Pittsburgh",
    "PIU": "Piura",
    "PIW": "Pikwitonei",
    "PJA": "Paso Jama",
    "PJC": "Pedro Juan Caballero",
    "PJG": "Panjgur",
    "PJI": "Puerto Jimenez",
    "PK1": "The Shandor Pass",
    "PK3": "The Karakoram Highway",
    "PK4": "Fairy Meadows",
    "PKB": "Parkersburg / Marietta",
    "PKC": "Petropavlovsk-Kamchatsky",
    "PKE": "Parkes",
    "PKH": "Punakha",
    "PKR": "Pokhara",
    "PKU": "Pekanbaru",
    "PKZ": "Pakse",
    "PKZ1": "Don Khong Island",
    "PKZ2": "Champassak",
    "PKZ3": "Tad Lo",
    "PKZ4": "Dong Kralor Border",
    "PKZ5": "Don Daeng Island ",
    "PL": "Palitana",
    "PLA": "Lake Placid, Ny",
    "PLB": "Plattsburgh",
    "PLJ": "Placencia",
    "PLM": "Palembang",
    "PLN": "Pellston",
    "PLO": "Port Lincoln",
    "PLS": "Providenciales",
    "PLU": "Belo Horizonte /Belo Horizon",
    "PLW": "Palu",
    "PLY": "Plymouth",
    "PLZ": "Peulla",
    "PMC": "Puerto Montt",
    "PMD": "Palmdale",
    "PMG": "Ponta Pora",
    "PMI": "Palma De Mallorca",
    "PMO": "Pismo Beach, Ca",
    "PMP": "Pampas",
    "PMR": "Palmerston North",
    "PMS": "Palmyra",
    "PMT": "Perito Moreno Town",
    "PMV": "Porlamar (Margarita)",
    "PMY": "Puerto Madryn",
    "PNA": "Pamplona",
    "PNC": "Ponca City",
    "PNCB": "Pantanal Corumba",
    "PND": "Punta Gorda",
    "PNH": "Phnom Penh",
    "PNH1": "Kompong Cham",
    "PNI": "Pohnpei",
    "PNJ": "Panjim",
    "PNK": "Pontianak",
    "PNL": "Pantelleria",
    "PNN": "Pantanal North",
    "PNQ": "Poona",
    "PNS": "Pensacola",
    "PNT": "Puerto Natales",
    "PNY": "Pondicherry",
    "PNZ": "Petrolina",
    "POA": "Porto Alegre",
    "POG": "Port Gentil",
    "POI": "Potosi",
    "POK": "Paracas",
    "POM": "Pompano Beach",
    "POR": "Portland",
    "POS": "Port Of Spain",
    "POU": "Poughkeepsie",
    "POZ": "Poznan",
    "PPA": "Praia da Pipa",
    "PPG": "Pago Pago",
    "PPL": "Papallacta",
    "PPN": "Popayan",
    "PPS": "Puerto Princesa",
    "PPT": "Papeete",
    "PQC": "Phu Quoc Island",
    "PQI": "Presque Isle",
    "PQM": "Palenque",
    "PQQ": "Port Macquarie",
    "PRC": "Prescott",
    "PRG": "Prague",
    "PRI": "Praslin Island",
    "PRK": "Park City Ut",
    "PRO": "Providence",
    "PRS": "Portrush",
    "PRV": "Providencia",
    "PRYR": "Periyar",
    "PSA": "Pisa",
    "PSE": "Ponce",
    "PSG": "Petersburg",
    "PSH": "Poshina",
    "PSK": "Pushkar",
    "PSM": "Playa Samara",
    "PSO": "Pasto",
    "PSP": "Palm Springs",
    "PSR": "Pescara",
    "PSS": "Posadas",
    "PST": "Phoenix/Scottsdale/T",
    "PSY": "Port Stanley",
    "PSZ": "Puerto Suarez",
    "PTA": "Port Angeles",
    "PTF": "Malololailai",
    "PTG": "Pietersburg",
    "PTH": "Perth Scotland",
    "PTI": "Paraty",
    "PTL": "Portland, Me",
    "PTO": "Phong Tho",
    "PTP": "Pointe A Pitre",
    "PTS": "Pantanal South",
    "PTY": "Panama City",
    "PUB": "Pueblo",
    "PUJ": "Punta Cana",
    "PUN": "Puno",
    "PUP": "Puerto Plata",
    "PUQ": "Punta Arenas",
    "PUR": "Purmamarca",
    "PUS": "Busan",
    "PUT": "Putao",
    "PUW": "Pullman",
    "PUX": "Puerto Varas",
    "PUX1": "Pusan",
    "PUZ": "Puerto Cabezas",
    "PVA": "Peninsula Valdes",
    "PVC": "Provincetown",
    "PVH": "Porto Velho",
    "PVR": "Puerto Vallarta",
    "PVU": "Provo",
    "PWM": "Portland Maine",
    "PWT": "Bremerton",
    "PXM": "Puerto Escondido",
    "PXO": "Porto Santo",
    "PXU": "Pleiku",
    "PXU1": "Lak Lake",
    "PYB": "Jeypore",
    "PYC": "Playa Conchal",
    "PYH": "Puerto Ayacucho",
    "PYO": "Puerto Ayora",
    "PYY": "Puerto Maya",
    "PZC": "Patzcuaro",
    "PZE": "Penzance",
    "PZO": "Puerto Ordaz",
    "PZZ": "Puerto Perez",
    "QAD": "Qadisha Valley",
    "QAZV": "Qazv",
    "QBF": "Vail/Eagle",
    "QCH": "Quechee",
    "QDR": "Deraa/Ramtha Border",
    "QDU": "Dusseldorf",
    "QDY": "Andong",
    "QJX": "Nong Khai",
    "QKB": "Breckenridge",
    "QKW": "Kanazawa",
    "QNG": "Nagano",
    "QNG1": "Yudanaka Onsen",
    "QNG2": "Kusatsu",
    "QOR": "Ordu",
    "QOR1": "Unye",
    "QRO": "Queretaro",
    "QUC": "Puerto La Cruz",
    "QUE": "Quetzaltenango",
    "RAB": "Rabaul",
    "RAC": "Racine, Wi",
    "RAJ": "Rajkot",
    "RAK": "Marrakech",
    "RAK1": "Essaouira",
    "RAK2": "The High Atlas and Imlil",
    "RAK4": "Erfoud",
    "RAK5": "Tizi Oussem",
    "RAK6": "N'azzadene Valley",
    "RAK7": "Ammslane",
    "RAK8": "Ijoukak",
    "RAK9": "Tizziane",
    "RAM": "Ramathra",
    "RAN": "Ranthambore",
    "RAO": "Riberao Preto",
    "RAP": "Rapid City",
    "RAR": "Raratonga",
    "RAS": "Rasht",
    "RAS1": "Ramsar",
    "RAS2": "Bandare Anzali and the Caspian",
    "RAW": "Rawatbhata",
    "RBA": "Rabat",
    "RBE": "Rattanakiri",
    "RBR": "Rio Branco",
    "RCH": "Riohacha",
    "RCM": "Co.Roscommon",
    "RDD": "Redding",
    "RDG": "Reading",
    "RDU": "Raleigh",
    "REC": "Recife",
    "RED": "Redondo Beach",
    "REG": "Reggio Calabria",
    "REL": "Trelew",
    "REP": "Siem Reap (Angkor Temples)",
    "REP1": "Ream National Park",
    "RES": "Resistencia",
    "REU": "Reus",
    "REW": "Rewa",
    "REX": "Reynosa",
    "REY": "Reykjavik",
    "RFBZ": "Rainforest Belize",
    "RFD": "Rockford",
    "RGA": "Rio Grande",
    "RGH": "Balurghat",
    "RGL": "Rio Gallegos",
    "RGN": "Yangon",
    "RHH": "Rio Hondo",
    "RHI": "Rhinelander",
    "RHO": "Rhodes",
    "RIC": "Richmond Va",
    "RIH": "Ruhaybat",
    "RIN1": "Ringmo",
    "RIO": "Rio de Janeiro",
    "RIS": "Rishikesh",
    "RIW": "Riverton",
    "RIX": "Riga",
    "RJA": "Rajahmundry",
    "RJI": "Rajouri",
    "RK": "Rann of Kutch",
    "RKD": "Rockland",
    "RKM": "Rocky Mount",
    "RKS": "Rock Springs",
    "RLT": "Arlit",
    "RMA": "Roma",
    "RMD": "Ramagundam",
    "RNB": "Ronneby",
    "RNK": "Rann of Kutch ",
    "RNO": "Reno Nv",
    "RNP": "Ruaha National Park",
    "RNS": "Rennes",
    "RO1": "Brasov",
    "RO2": "The Carpathians",
    "ROA": "Roanoke",
    "ROC": "Rochester",
    "ROH": "Rohet",
    "ROI": "Coroico",
    "ROM": "Rome",
    "ROP": "Rota",
    "ROR": "Koror",
    "ROS": "Rosario",
    "ROV": "Rostov",
    "ROW": "Roswell",
    "RPR": "Raipur",
    "RRG": "Rodrigues Island",
    "RRK": "Rourkela",
    "RSA": "Santa Rosa",
    "RSC": "Roscoff",
    "RSD": "Rock Sound",
    "RSL": "Rosslaire",
    "RSU": "Yosu",
    "RTB": "Roatan",
    "RTC": "Ratnagiri",
    "RTM": "Rotterdam",
    "RU2": "Ferghana Valley",
    "RUH": "Riyadh",
    "RUI": "Ruidoso",
    "RUN": "St-Denis De La Reunion",
    "RUP": "Rupsi",
    "RUT": "Rutland",
    "RUY": "Copan",
    "RWP": "Rawalpindi",
    "RXX": "Riobamba",
    "SAB": "Saba Island",
    "SAF": "Santa Fe",
    "SAG": "Sagaing",
    "SAH": "Sana'a",
    "SAH1": "Marib",
    "SAH2": "Manakha",
    "SAL": "San Salvador",
    "SAM": "Samaipata",
    "SAN": "San Diego",
    "SAO": "Sao Paulo",
    "SAP": "San Pedro Sula",
    "SAP1": "Xa Pho",
    "SAP2": "Sin Chai",
    "SAP3": "Thanh Phu",
    "SAQ": "San Andros / Andros Island",
    "SAR": "Stranraer",
    "SAT": "San Antonio",
    "SAV": "Savannah",
    "SBA": "Santa Barbara",
    "SBH": "St Barthelemy",
    "SBJ": "Samdrup Jongkhar",
    "SBL": "Sanibel Island",
    "SBN": "South Bend",
    "SBS": "Steamboat Springs",
    "SBU": "South Burlington",
    "SBW": "Sibu",
    "SCC": "San Cristobal",
    "SCE": "State College",
    "SCL": "Santiago ",
    "SCN": "Saarbruecken",
    "SCQ": "Santiago De Compostela",
    "SCR": "San Cristobal",
    "SCT": "Socotra",
    "SCU": "Santiago De Cuba",
    "SCV": "Sacred Valley",
    "SD1": "The Sahara",
    "SDD": "Louisville",
    "SDE": "Santiago Del Estero",
    "SDF": "Louisville",
    "SDJ": "Sendai",
    "SDK": "Sandakan",
    "SDK1": "Sepilok",
    "SDK2": "Kinabatangan",
    "SDK3": "Lankayan Island",
    "SDK4": "Selingan Island",
    "SDL": "Sundsvall",
    "SDN": "Sandane",
    "SDO": "San Pedro",
    "SDQ": "Santo Domingo",
    "SDR": "Santander",
    "SDY": "Sidney",
    "SEA": "Seattle",
    "SEB": "Sebha",
    "SEB2": "Germa",
    "SEB3": "Dawada Lakes",
    "SEB4": "Lake Gabraoun",
    "SEB5": "Acacus Desert",
    "SEB6": "Al Awinat",
    "SEB7": "Ghat",
    "SED": "Sedona",
    "SEL": "Seoul",
    "SEL1": "Seoraksan National Park",
    "SEL2": "Geumgangsan Diamond Mountains",
    "SEM": "Simien Mountains",
    "SEW": "Siwa Oasis ",
    "SEZ": "Mahe Island",
    "SFA": "Sfax",
    "SFD": "San Fernando Apure",
    "SFG": "St Martin",
    "SFJ": "Kangerlussuaq",
    "SFN": "Santa Fe",
    "SFO": "San Francisco Ca",
    "SFT": "Skelleftea",
    "SGA": "San Agustin",
    "SGC": "Surgut",
    "SGD": "Sonderborg",
    "SGF": "Springfield  Mo",
    "SGN": "Ho Chi Minh City",
    "SGN1": "Can Tho (Mekong Delta)",
    "SGN2": "Chau Doc",
    "SGN3": "Mui Ne",
    "SGN4": "Trai Mat",
    "SGN5": "Nam Cat Tien National Park",
    "SGNP": "Serengeti National Park",
    "SGO": "St George",
    "SGR": "Selous Game Reserve",
    "SGU": "Saint George",
    "SGY": "Skagway",
    "SHA": "Shanghai",
    "SHA3": "Tai An",
    "SHA4": "Qufu",
    "SHC": "Indaselassie",
    "SHD": "Staunton",
    "SHE": "Sheiyang",
    "SHI": "Shimla",
    "SHJ": "Sharjah",
    "SHL": "Shillong Barapani",
    "SHO": "Sokcho",
    "SHP": "Shahpura",
    "SHR": "Sheridan",
    "SHV": "Shreveport",
    "SID": "Sal",
    "SII": "Shaxi",
    "SIL": "Sitla",
    "SIM": "San Simeon",
    "SIN": "Singapore",
    "SIP": "Simferopol",
    "SIT": "Sitka",
    "SJB1": "Samdrup Jongkhar",
    "SJC": "San Jose",
    "SJD": "San Jose Del Cabo",
    "SJF": "St John Island",
    "SJJ": "Sarajevo",
    "SJO": "San Jose",
    "SJT": "San Angelo",
    "SJU": "San Juan",
    "SJW": "Shijiazhuang",
    "SKB": "St Kitts",
    "SKC": "Sequoia & Kings Nat",
    "SKD": "Samarkand",
    "SKG": "Thessaloniki",
    "SKO": "Skoura",
    "SKP": "Skopje",
    "SKS": "Vojens Lufthavn",
    "SKT": "Skeleton Coast",
    "SLA": "Salta",
    "SLC": "Salt Lake City",
    "SLK": "Saranac Lake",
    "SLL": "Salalah",
    "SLN": "Salina",
    "SLNP": "South Luangwa National Park",
    "SLO": "San Luis Obispo",
    "SLP": "San Luis Potosi",
    "SLS": "Salisbury",
    "SLU": "Saint Lucia",
    "SLV": "Silver Springs",
    "SLW": "Saltillo",
    "SLZ": "Sao Luis",
    "SMA": "Santa Maria",
    "SMF": "Sacramento",
    "SML": "Stella Maris",
    "SMM": "Semporna",
    "SMO": "Santa Monica",
    "SMR": "Santa Marta",
    "SMS": "St Marie",
    "SNA": "Santa Ana",
    "SNC": "San Carlos",
    "SNN": "Shannon",
    "SNO": "Sakon Nakhon",
    "SNP": "Shenandoah Nat Park",
    "SNR": "Stranraer",
    "SNV": "Santa Elena",
    "SOF": "Sofia",
    "SOG": "Sogndal",
    "SOL": "Solvang",
    "SOM": "San Tome",
    "SON": "Son La",
    "SOP": "Southern Pines",
    "SOU": "Southampton",
    "SOW": "Show Low",
    "SPC": "Santa Cruz La Palma",
    "SPD": "Saidpur",
    "SPG": "Springfield",
    "SPK": "Sapporo",
    "SPK1": "Noboribetsu",
    "SPN": "Saipan",
    "SPN1": "Shey Phoksundo National Park",
    "SPO": "South Portland",
    "SPP": "Sipidan",
    "SPQ": "San Pedro de Quemez",
    "SPR": "San Pedro Ambergris Caye",
    "SPS": "Wichita Falls",
    "SPW": "Spencer",
    "SRE": "Sucre",
    "SRP": "Sarapiqui",
    "SRQ": "Sarasota",
    "SRZ": "Santa Cruz",
    "SS2": "Hadibo",
    "SSA": "Salvador De Bahia",
    "SSB": "St Croix Island",
    "SSD": "Seaside",
    "SSE": "Sholapur",
    "SSG": "Malabo",
    "SSH": "Sharm El Sheikh",
    "SSH1": "Nuweiba",
    "SSH2": "Dahab",
    "SSH3": "Sinai and St. Catherine",
    "SSH4": "El Gouna",
    "SSJ": "Sandnessjoen",
    "SSO": "Silver Springs/Ocala",
    "SSQ": "La Sarre",
    "SSX": "Samsun",
    "SSX1": "Amasya",
    "STC": "Santa Clara",
    "STD": "Santo Domingo",
    "STG": "Strasburg",
    "STI": "Santiago",
    "STL": "St Louis",
    "STM": "Santarem",
    "STN": "Satpura National Park",
    "STO": "Stowe",
    "STP": "St Paul",
    "STR": "Stuttgart",
    "STT": "St Thomas Island",
    "STU": "Sturbridge",
    "STV": "Surat",
    "STX": "St Croix Island",
    "STZ": "Santa Cruz",
    "SUB": "Surabaya",
    "SUF": "Lamezia Terme",
    "SUH": "Sur",
    "SUH1": "Ras Al Hadd",
    "SUH2": "Wahiba Sands",
    "SUK1": "Donghae",
    "SUL": "Sungai Mawang ",
    "SUN": "Sunderland",
    "SUP": "Costa do Sauipe",
    "SUV": "Suva",
    "SUX": "Sioux City",
    "SVD": "St Vincent",
    "SVG": "Stavanger",
    "SVL": "Savonlinna",
    "SVQ": "Sevilla",
    "SVU": "Savusavu",
    "SVX": "Ekaterinburg",
    "SWA": "Shantou",
    "SWF": "Newburgh/Poughkeepsie",
    "SWM": "Sawai Madhopur",
    "SWP": "Swakopmund",
    "SWQ": "Sumbawa Island",
    "SXB": "Strasbourg",
    "SXL": "C0.Sligo",
    "SXM": "St Maarten",
    "SXR": "Srinagar",
    "SY1": "Rasafa",
    "SY2": "Tadmor",
    "SYD": "Sydney",
    "SYM": "Simao",
    "SYO": "Shonai",
    "SYR": "Syracuse",
    "SYX": "Sanya",
    "SYY": "Stornoway",
    "SYZ": "Shiraz",
    "SZG": "Salzburg",
    "SZV": "Suzhou",
    "SZX": "Shenzhen",
    "SZZ": "Szczecin",
    "TAB": "Tobago",
    "TAC": "Tacloban",
    "TAD": "Tadjourah",
    "TAE": "Daegu",
    "TAG": "Tagbilaran",
    "TAI": "Ta'izz",
    "TAI2": "Tarim",
    "TAI3": "Wadi Dhahr",
    "TAK": "Takab",
    "TAM": "Tampico",
    "TAO": "Taos",
    "TAP": "Tapachula",
    "TAS": "Tashkent",
    "TAS1": "Margilan",
    "TAS2": "Kokand",
    "TAS3": "Chernyaevka Border",
    "TAT": "Tatry/Poprad",
    "TAX": "Taxco",
    "TAY": "Tayrona National Park",
    "TAZ": "Dashoguz",
    "TAZ1": "Konye Urgench",
    "TAZ2": "Hodjaili Border",
    "TBN": "Fort Leonard Wood",
    "TBP": "Tumbes",
    "TBS": "Tbilisi",
    "TBS1": "Gudauri",
    "TBS2": "Bakuriani",
    "TBS3": "Telavi",
    "TBS4": "Kazbegi",
    "TBS5": "Becho (Svaneti Region)",
    "TBS6": "Mestia",
    "TBS7": "Ushguli",
    "TBS8": "Sadakhlo Border",
    "TBS9": "Akhalsikhe",
    "TBT": "Tabatinga",
    "TBU": "Nuku Alofa/Tongatapu",
    "TBZ": "Tabriz",
    "TBZ1": "Jolfa",
    "TCC": "Tacarigua",
    "TCH": "Tachileik",
    "TCO": "Trancoso",
    "TCQ": "Tacna",
    "TCR": "Turrialba",
    "TDA": "Trinidad",
    "TDD": "Trinidad",
    "TDP": "Torres Del Paine",
    "TDX": "Trat",
    "TDX1": "Koh Chang",
    "TED": "Thisted",
    "TEI": "Tezu",
    "TEM": "Templepatrick",
    "TEN": "Tena",
    "TER": "Terceira Island",
    "TEX": "Telluride",
    "TEZ": "Tezpur",
    "TFF": "Tefe",
    "TFN": "Tenerife North",
    "TFS": "Tenerife South",
    "TGA": "Taganga",
    "TGD": "Podgorica",
    "TGG": "Kuala Terengganu",
    "TGM": "Transylvania",
    "TGU": "Tegucigalpa",
    "TGZ": "Tuxtla Gutierrez",
    "TH1": "Pakbeng",
    "THA": "Thalassery",
    "THE": "Teresina",
    "THP": "Thimpu",
    "THR": "Tehran",
    "THR1": "Kashan",
    "THS": "Sukhothai",
    "THS1": "Kamphaeng Phet",
    "THU": "Pituffik",
    "THZ": "Tahoua",
    "TIA": "Tirana",
    "TIJ": "Tijuana",
    "TIK": "Tiskita",
    "TIL1": "Tilcara",
    "TIN": "Tineghir",
    "TIP": "Tripoli",
    "TIP1": "Leptis Magna",
    "TIP2": "Ghadames",
    "TIP3": "Gharyan",
    "TIQ": "Tinian",
    "TIR": "Tirupati",
    "TIS": "Thursday Island",
    "TIT": "Titusville",
    "TIZ": "Tari",
    "TJ1": "Penjikent",
    "TJ2": "Wakhan Corridor",
    "TJ4": "Khorog",
    "TJ5": "Iskandar Kol",
    "TJA": "Tarija",
    "TJM": "Tyumen",
    "TJV": "Thanjavur",
    "TKA": "Takerkiba",
    "TKK": "Truk",
    "TKL": "Tikal",
    "TKQ": "Kigoma",
    "TKS": "Tokushima",
    "TKU": "Turku",
    "TLA": "Tulum Archaeological Site ",
    "TLH": "Tallahassee",
    "TLL": "Tallinn",
    "TLS": "Toulouse",
    "TLU": "Tuscaloosa",
    "TLV": "Tel Aviv",
    "TM1": "Murghab",
    "TM2": "Kugitag Nature Reserve",
    "TM3": "Koyten Dag Dinosaur Plateau",
    "TM4": "Karlyuk Caves",
    "TMC": "Tanzania Mainland Coast",
    "TMD": "Tamarindo",
    "TMJ": "Termez",
    "TMP": "Tampere",
    "TMR": "Tamanrasset Aguemar",
    "TMW": "Tamworth",
    "TNA": "Jinan",
    "TND": "Trinidad",
    "TNF": "Turneffe Islands",
    "TNG": "Tangier",
    "TNG1": "Chefchaouen",
    "TNG2": "Aazib Tamsoult",
    "TNI": "Satna",
    "TNP": "Tarangire National Park",
    "TNR": "Antananarivo",
    "TNX": "Stung Treng",
    "TNZ": "Tosontsengel",
    "TNZ1": "White Lake",
    "TNZ2": "Tsetserleg",
    "TOB": "Tobruk",
    "TOB1": "Ummsaad Border",
    "TOD": "Tioman Islands",
    "TOL": "Toledo",
    "TOM": "Timbuctoo",
    "TON": "Tonpah",
    "TOR": "Troon",
    "TOS": "Tromso",
    "TOS1": "Vengsoy Island",
    "TOY": "Toyama",
    "TPA": "Tampa",
    "TPC": "Tarapoa",
    "TPE": "Taipei",
    "TPH": "Tonopah Nv",
    "TPL": "Temple",
    "TPP": "Tarapoto",
    "TPQ": "Tepic",
    "TPS": "Trapani",
    "TPZ": "Tupiza",
    "TR1": "Egirdir",
    "TR2": "Cappadocia",
    "TR3": "Kasimlar",
    "TR4": "Asar Tepe",
    "TR5": "Selge",
    "TR6": "Kasnak Forest",
    "TR7": "Adada",
    "TR8": "Gallipoli",
    "TRA": "Tralee",
    "TRC": "Torreon",
    "TRD": "Tiradentes",
    "TRE": "Tiree",
    "TRG": "Tauranga",
    "TRI": "Bristol/Johnson City/Kingsport",
    "TRK": "Tarakan",
    "TRL": "Tres Lagos",
    "TRN": "Turin",
    "TRO": "Trongsa",
    "TRR": "Trincomalee",
    "TRS": "Trieste",
    "TRU": "Trujillo",
    "TRV": "Thiruvananthapuram",
    "TRZ": "Tiruchirapally",
    "TSA": "Taipei",
    "TSE": "Astana",
    "TSR": "Timisoara",
    "TSS": "Tsagaan Suvarga ",
    "TSV": "Townsville",
    "TTE": "Ternate",
    "TTJ": "Tottori",
    "TTN": "Trenton, Usa",
    "TTQ": "Tortuguero",
    "TU1": "Tusheti",
    "TUC": "Tucuman",
    "TUF": "Tours",
    "TUL": "Tulsa",
    "TUN": "Tunis",
    "TUO": "Taupo",
    "TUP": "Tupelo",
    "TUS": "Tucson",
    "TUY": "Tulum",
    "TVC": "Traverse City",
    "TVF": "Thief River Falls",
    "TVL": "South Lake Tahoe",
    "TWB": "Toowoomba",
    "TWF": "Twin Falls",
    "TWU": "Tawau",
    "TXK": "Texarkana",
    "TXN": "Tunxi",
    "TYN": "Taiyuan",
    "TYN1": "Pingyao",
    "TYN2": "Wutaishan",
    "TYO": "Tokyo",
    "TYO1": "Hakone",
    "TYO2": "Takayama",
    "TYO3": "Tsumago",
    "TYO4": "Shimoda",
    "TYO5": "Numazu",
    "TYO6": "Nikko",
    "TYR": "Tyler",
    "TYS": "Knoxsville",
    "TZA": "Belize Municipal",
    "TZN": "South Andros",
    "TZX": "Trabzon (Black Sea)",
    "UAG": "Aghadowey",
    "UAK": "Narsarsuaq",
    "UAN": "Annalong",
    "UAQ": "San Juan",
    "UBA": "Uberaba",
    "UBC": "Ballycastle",
    "UBG": "Bangor",
    "UBH": "Bushmills",
    "UBJ": "Ube",
    "UBM": "Ballymena",
    "UBP": "Ubon Ratchathani",
    "UBP1": "Chong Mek Border",
    "UCA": "Utica",
    "UCF": "Crawfordsburn",
    "UCL": "Carnlough",
    "UCO": "Cookstown",
    "UCP": "Campsie",
    "UDI": "Uberlandia",
    "UDR": "Udaipur",
    "UDU": "Dunadry",
    "UEN": "Enniskillen",
    "UET": "Quetta",
    "UGC": "Urgench",
    "UGC1": "Khiva",
    "UGC2": "Tashauz Border",
    "UGC3": "Nurata",
    "UIB": "Quibdo",
    "UIN": "Quincy",
    "UIO": "Quito",
    "UIP": "Quimper",
    "UJJ": "Ujjain",
    "UKA": "Akumal",
    "UKY": "Kyoto",
    "UKY1": "Mount Koya",
    "ULD": "Londonderry",
    "ULK": "Lisnaskea",
    "ULN": "Ulaanbaatar",
    "ULN1": "Hustain Nuruu",
    "ULN2": "Terelj National Park",
    "ULN3": "Tuul River Valley",
    "ULU": "Ulu Ai",
    "ULV": "Limavady",
    "ULY": "Ulyanovsk",
    "UMA": "Maghera",
    "UME": "Umea",
    "UMO": "Omagh",
    "UMR": "Woomera",
    "UNB": "Newtownabbey",
    "UNE": "Newry",
    "UNT": "Unst Shetland Islands",
    "UNW": "Newcastle",
    "UPG": "Ujung Pandang",
    "UPN": "Uruapan",
    "UPO": "Portaferry",
    "URC": "Urumqi",
    "URC1": "Turpan",
    "URO": "Rouen",
    "URU": "Urubamba Valley",
    "USH": "Ushuaia",
    "USM": "Koh Samui",
    "USN": "Ulsan",
    "UST": "St.Augustine",
    "UTH": "Udon Thani",
    "UTI": "Utica, Ny",
    "UTN": "Upington",
    "UUD": "Ulan-Ude",
    "UUP": "Upperlands",
    "UVF": "St Lucia",
    "UXM": "Uxmal",
    "UYU": "Uyuni and Salt Flats",
    "VAA": "Vaasa",
    "VAD": "Valladolid",
    "VAI": "Vail. Co",
    "VAN": "Van",
    "VAP": "Valparaiso",
    "VAR": "Varna",
    "VARA": "Varanda",
    "VAS": "Sivas",
    "VBE": "Vero Beach",
    "VBY": "Visby",
    "VCE": "Venice",
    "VCQ": "Vichuquen",
    "VCS": "Con Dao Island",
    "VDA": "Ovda",
    "VDL": "Villa De Leyva",
    "VDM": "Viedma",
    "VDS": "Vadso",
    "VDZ": "Valdez",
    "VEN": "Venice, Fl",
    "VER": "Veracruz",
    "VEY": "Vestmannaeyjar",
    "VGA": "Vijayawada",
    "VIE": "Vienna",
    "VIJ": "Virgin Gorda",
    "VIL": "Villaz—n",
    "VIN": "Varkala",
    "VIR": "Virginia Beach",
    "VIS": "Visalia",
    "VIX": "Vitoria",
    "VKS": "Vicksburg",
    "VLA": "Villa La Angostura",
    "VLC": "Vilanculos",
    "VLD": "Valdosta",
    "VLG": "Villa Gesell",
    "VLI": "Port Vila",
    "VLN": "Valencia",
    "VLV": "Valera",
    "VNA": "Vinales",
    "VNO": "Vilnius",
    "VNS": "Varanasi",
    "VNT": "Ventura",
    "VNV": "Valle Nevado",
    "VOG": "Volgograd",
    "VRA": "Varadero",
    "VRI": "Villarica",
    "VRK": "Varkaus",
    "VRN": "Verona",
    "VSA": "Villahermosa",
    "VSG": "Lugansk",
    "VST": "Vasteras",
    "VTA": "Ventura Beach",
    "VTE": "Vientiane",
    "VTE1": "Vang Vieng",
    "VTZ": "Vishakhapatnam",
    "VUP": "Valledupar",
    "VVO": "Vladivostok",
    "VWY": "The Nilgiris Hills",
    "VXO": "Vaxjo",
    "WAD": "Wadi Matkhandoush ",
    "WAK": "Wakhan Corridor",
    "WAL": "Walsall",
    "WAM": "Wadi Mujib",
    "WAR": "Warwick",
    "WAS": "Washington",
    "WAT": "Waterford",
    "WAW": "Warsaw",
    "WAY": "Wayanad",
    "WBU": "Boulder",
    "WCT": "Wine Country",
    "WDG": "Enid",
    "WEL": "Welwyn",
    "WES": "Co.Westmeath",
    "WEX": "Co.Wexford",
    "WGA": "Wagga-Wagga",
    "WGC": "Warrangal",
    "WGE": "Walgett",
    "WHW": "West Hollywood",
    "WIC": "Co.Wicklow",
    "WIL": "Wilmington",
    "WIN": "Winnemucca",
    "WLG": "Wellington",
    "WLM": "Williams",
    "WMH": "Mountain Home",
    "WMT": "White Mountains",
    "WNG": "Wangdue",
    "WPT": "Williamsport",
    "WRE": "Whangarei",
    "WRG": "Wrangell",
    "WRL": "Worland",
    "WRO": "Wroclaw",
    "WSA": "Winston-Salem",
    "WST": "Westerly",
    "WSZ": "Westport",
    "WTN": "Williamstown",
    "WUH": "Wuhan",
    "WVB": "Walvis Bay",
    "WWK": "Wewak",
    "WYA": "Whyalla",
    "XAW": "Capreol",
    "XAZ": "Cambellton",
    "XBR": "Brockville",
    "XCI": "Chambord",
    "XCM": "Chatham",
    "XDL": "Chandler",
    "XDM": "Drummondville",
    "XDU": "Hervey",
    "XEE": "Lac Edouard",
    "XEH": "Ladysmith",
    "XEJ": "Langford",
    "XEK": "Melville",
    "XEL": "New Carlisle",
    "XEM": "New Richmond",
    "XFD": "Stratford",
    "XFE": "Parent",
    "XFG": "Perce",
    "XFL": "Shawinigan",
    "XFM": "Shawnigan",
    "XFO": "Taschereau",
    "XFQ": "Weymont",
    "XFS": "Alexandria",
    "XFU": "San Miguel de Allende",
    "XFV": "Brantford",
    "XGJ": "Cobourg",
    "XGK": "Coteau",
    "XGY": "Grimsby",
    "XHS": "Chemainus",
    "XIA": "Guelph",
    "XIB": "Ingersoll",
    "XID": "Maxville",
    "XIM": "Saint-Hyacinthe",
    "XIO": "St Marys",
    "XIP": "Woodstock",
    "XIY": "Xi'an",
    "XJL": "Joliette",
    "XJQ": "Jonquiere",
    "XKU": "Kusadasi",
    "XLZ": "Truro",
    "XMN": "Xiamen",
    "XNN": "Xining",
    "XNN1": "Koko Nor (Qinghai Lake)",
    "XOK": "Oakville",
    "XPB": "Parksville",
    "XPN": "Brampton",
    "XPX": "Pointe-Aux-Trembles",
    "XQP": "Quepos",
    "XQU": "Qualicum",
    "XRP": "Riviere-A-Pierre",
    "XRY": "Jerez",
    "XSC": "South Caicos",
    "XSZ": "Setubal",
    "XTY": "Strathroy",
    "XVV": "Belleville",
    "XWY": "Wyoming",
    "XZB": "Casselman",
    "XZC": "Glencoe",
    "YAI": "Chillan",
    "YAK": "Yakima",
    "YAM": "Sault Ste.Marie",
    "YAP": "Yap",
    "YAS": "Yashil Kul Lake",
    "YAZ": "Tofino",
    "YBA": "Banff",
    "YBC": "Baie Comeau",
    "YBL": "Campbell River",
    "YBR": "Blue River",
    "YBX": "Blanc Sablon",
    "YCA": "Courtenay",
    "YCC": "Cornwall",
    "YCD": "Nanaimo",
    "YCG": "Castlegar",
    "YCM": "St Catharines",
    "YDF": "Deer Lake",
    "YDN": "Dauphin",
    "YDQ": "Dawson Creek",
    "YEC": "Yechon",
    "YEG": "Edmonton",
    "YEL": "Yellowstone Nat Park",
    "YEV": "Inuvik",
    "YEY": "Amos",
    "YFB": "Iqaluit",
    "YFC": "Fredericton",
    "YFS": "Fort Simpson",
    "YGB": "Gillies Bay",
    "YGP": "Gaspe",
    "YGR": "Iles De La Madeleine",
    "YGT": "Igloolik",
    "YGW": "Kuujjuarapik",
    "YGX": "Gillam",
    "YHB": "Hudson Bay",
    "YHD": "Dryden",
    "YHI": "Holman Island",
    "YHN": "Hornepayne",
    "YHY": "Hay River",
    "YHZ": "Halifax",
    "YIB": "Atikokan",
    "YIH": "Yichang",
    "YIN": "Yining",
    "YIN1": "Korgass Border",
    "YIR": "Yira Alem",
    "YJA": "Jasper",
    "YJT": "Stephenville",
    "YK": "Yaxchilan",
    "YKA": "Kamloops",
    "YKF": "Kitchener",
    "YKL": "Schefferville",
    "YKQ": "Waskaganish",
    "YKS": "Yakutsk",
    "YLD": "Chapleau",
    "YLQ": "La Tuque",
    "YLW": "Kelowna",
    "YMM": "Fort Mcmurray",
    "YMO": "Moosonee",
    "YMQ": "Montreal",
    "YMT": "Chibougamau",
    "YND": "Gatineau",
    "YNG": "Youngstown",
    "YOJ": "High Level",
    "YOO": "Oshawa",
    "YOP": "Rainbow Lake",
    "YOW": "Ottawa",
    "YPE": "Peace River",
    "YPF": "Esquimalt",
    "YPR": "Prince Rupert /Princ Rupert",
    "YPW": "Powell River",
    "YPZ": "Burns Lake",
    "YQB": "Quebec City",
    "YQC": "Quaqtaq",
    "YQD": "The Pas",
    "YQG": "Windsor",
    "YQH": "Watson Lake",
    "YQI": "Yarmouth",
    "YQK": "Kenora",
    "YQL": "Lethbridge",
    "YQM": "Moncton",
    "YQQ": "Comox",
    "YQR": "Regina",
    "YQT": "Thunder Bay",
    "YQU": "Grande Prairie",
    "YQX": "Gander",
    "YQZ": "Quesnel",
    "YRB": "Resolute",
    "YRL": "Red Lake",
    "YSB": "Sudbury",
    "YSF": "Stony Rapids",
    "YSH": "Smith Falls",
    "YSJ": "Saint John  N.B",
    "YSL": "St Leonard",
    "YSM": "Fort Smith",
    "YSN": "Salmon Arm",
    "YTA": "Pembroke",
    "YTD": "Thicket Portage",
    "YTH": "Thompson",
    "YTS": "Timmins",
    "YUG": "Yungas",
    "YUJ": "Yasuj",
    "YUM": "Yuma",
    "YVA": "Moroni",
    "YVB": "Bonaventure",
    "YVC": "La Ronge",
    "YVP": "Kuujjuaq",
    "YVR": "Vancouver",
    "YVZ": "Deer Lake",
    "YWG": "Winnipeg",
    "YWK": "Wabush",
    "YWL": "Williams Lake",
    "YWR": "White River",
    "YXC": "Cranbrook",
    "YXE": "Saskatoon",
    "YXH": "Medicine Hat",
    "YXJ": "Fort St John",
    "YXL": "Sioux Lookout",
    "YXS": "Prince George",
    "YXT": "Terrace",
    "YXY": "Whitehorse",
    "YYB": "North Bay",
    "YYC": "Calgary",
    "YYD": "Smithers",
    "YYF": "Penticton",
    "YYG": "Charlottetown",
    "YYJ": "Victoria",
    "YYQ": "Churchill",
    "YYR": "Goose Bay",
    "YYT": "St.John    N.S",
    "YYU": "Kapuskasing",
    "YYY": "Mont Joli",
    "YYZ": "Toronto",
    "YZF": "Yellowknife",
    "YZT": "Port Hardy",
    "YZV": "Sept-Iles",
    "ZAC": "Zacatecas",
    "ZAG": "Zagreb",
    "ZAH": "Zahedan",
    "ZAL": "Valdivia",
    "ZAM": "Zamboanga",
    "ZAN": "Zanjan ",
    "ZAZ": "Zaragoza",
    "ZBV": "Vail/Eagle",
    "ZCO": "Temuco",
    "ZDJ": "Berne",
    "ZER": "Zero",
    "ZGI": "Gods River",
    "ZHA": "Zhanjiang",
    "ZHO": "Zhongwei",
    "ZIC": "Manzanillo",
    "ZIH": "Zihuatanejo",
    "ZIO": "Zion National Park",
    "ZLO": "Manzanillo",
    "ZMT": "Zermatt",
    "ZNA": "Nanaimo",
    "ZND": "Zinder",
    "ZNE": "Newman",
    "ZNZ": "Zanzibar",
    "ZNZ1": "Stone Town",
    "ZNZ2": "Zanzibar and Islands",
    "ZOS": "Osorno",
    "ZPC": "Pucon",
    "ZQN": "Queenstown",
    "ZRF": "Rockford",
    "ZRH": "Zurich",
    "ZSY": "Scottsdale",
    "ZTH": "Zakinthos",
    "ZYA": "Zapallar",
    "ZZO": "Zona Cafetera"
}



# Packaged, to be called via cli.py cli interface
def auto_connect() -> None:
    import random
    from typing import List
    from . import config
    from . import wg_peer
    from .wg_peer import WgPeer, connect_only_to

    # ==CODE==#
    config.load_config()

    import typing
    import requests

    import urllib3

    urllib3.disable_warnings()  # Suppress Insecure HTTPS warnings

    print("==API==")
    # print(f"key: {config.api_key}") # hidden for privacy
    # print(f"secret: {config.api_secret}") # hidden for privacy
    print(f"Going to {config.opnsense_api}")

    # API
    session = requests.Session()
    session.auth = (config.api_key, config.api_secret)

    # API check
    try:
        response = session.get(f"{config.opnsense_api}/wireguard/client/list_servers", verify=False)
        response_json = response.json()
        if response_json["status"] != "ok":
            raise Exception(f"JSON response is not OK! ({response_json})")
    except Exception as e:
        print(
            f"[ERR] Something went wrong testing the API. Check your network connection, then make sure the API key is correct and it has permissions to management wireguard. ({e})")
        exit(-1)

    ifname = ""  # ifname will be automatically found via searchClient

    print("=== === OPNSpeedGuard === ===")
    if config.automatic:
        print("[INFO] RUNNING IN AUTOMATIC MODE. ALL USER INPUT WILL BE EMULATED.")
    # == USER CONFIG ==
    print("== == ==")

    disable_first = config.get_do_disable_first()
    randomize = config.get_do_randomize()

    print("== == ==")
    if not config.wait_for_user_start():
        exit()

    print("=== START ===")
    peers: typing.Dict[str, WgPeer] = {}


    # === Find all peers ===

    print("Finding all peers...")

    request_json = {
        "current": 1,
        "rowCount": -1,
        "servers": [config.instance_UUID],
        "sort": {},
        "searchPhrase": ""
    }

    output = session.post(f"{config.opnsense_api}/wireguard/client/searchClient/", json=request_json, verify=False)
    output_json = output.json()

    for peer_js in output_json["rows"]:
        if peer_js["servers"] != config.instance_UUID:
            raise Exception("[!] OHNON!O!NO!NO!")
        peer = WgPeer(peer_js)
        peers[peer.pubkey] = peer


    # === DISABLE ===

    # TODO: This is not actually required anymore I think...
    if disable_first:
        print("=== DISABLING PEERS... ===")

        enabled_peers = [peer for peer in peers.values() if peer.enabled]
        for idx, peer in enumerate(enabled_peers):
            if config.verbose:
                print(f"Disabling {peer.name} ({int((idx + 1) / len(enabled_peers) * 100)}%)...")
            peer.disable(session)

        print("---")
        print("Applying...")
        wg_peer.wg_apply(session)


    # === ENABLE ===

    print("=== ENABLING PEERS... ===")

    for idx, peer in enumerate(peers.values()):
        if config.verbose:
            print(f"Enabling {peer.name} ({int((idx + 1) / len(peers) * 100)}%)...")
        peer.enable(session, randomize)


    print("---")
    print("Applying...")
    wg_peer.wg_apply(session)


    # === INFO ===

    print("=== STATUS ===")

    request_json = {
        "current": 1,
        "rowCount": -1,
        "sort": {}
    }
    output = session.post(f"{config.opnsense_api}/wireguard/service/show", json=request_json, verify=False)
    status_json = output.json()

    idx = 1
    candidates: typing.List[WgPeer] = []

    for item in status_json["rows"]:
        if item["type"] == "peer":

            if item["public-key"] not in peers:  # probably other interfaces
                continue

            p = peers[item["public-key"]]
            p.add_connection_info(item)
            if p.online:
                print(f"{idx}: {p.get_human_info()}")
                idx += 1
                candidates.append(p)

    # -- SELECTION --

    selection_idx = config.get_peer_selection(len(candidates))
    selected_peer = candidates[selection_idx]

    print(f"Selected [{selection_idx + 1}] {selected_peer.name}")

    # === CONNECT ===
    # Disconnect all but selection

    print(f"=== Disconnecting all but {selected_peer.name}... ===")

    connect_only_to(session, peers.values(), selected_peer)

    print("---")
    print("Applying...")
    wg_peer.wg_apply(session)


    # === SPEEDTEST ===

    print("=== Speedtest autoconnect? (Optional) ===")
    if config.get_do_speedtest():

        # == DO SPEEDTEST ==
        from . import speedtest_utils

        speed_satisfied = False
        speedtest_candidates: List[WgPeer] = []

        # Test original one first.
        speedtest_current_candidate = selected_peer

        print(f"Testing {speedtest_current_candidate.name}...")
        speed = speedtest_utils.speedtest()
        print(f"{speedtest_current_candidate.name} (Down): {speedtest_utils.format_bps(speed)}")
        speedtest_current_candidate.set_speed(speed)

        # Add candidate, remove from random selection
        speedtest_candidates.append(speedtest_current_candidate)
        candidates.remove(speedtest_current_candidate)

        if speed > config.get_speedtest_threshold():
            print("Speed threshold satisfied.")
            speed_satisfied = True
        else:
            print("Speed threshold not satisfied. Testing another peer...")

        random.shuffle(candidates)

        # The logic here is kind of messy as the first (user-selected) peer has to be handled manually, but it also has to
        # counted for candidate selection and best-effort connection if no peer can reach the threshold. Hopefully there are no bugs :/
        # Also the peer enable states are juggled around since I don't want to waste API latency on redundant disabling/enabling peers, but it's kind of dangerous.
        for i in range(1, config.speedtest_maxtries):
            print("---")
            if speed_satisfied: # Loop passthrough from selected peer above
                break

            if len(candidates) == 0:
                # No more to find
                print("[W] Out of peers to test.")
                break

            # New peer
            speedtest_current_candidate = candidates.pop(0)
            speedtest_candidates.append(speedtest_current_candidate)

            print(f"Speedtest candidate #{i}/{config.speedtest_maxtries}: {speedtest_current_candidate.name}")
            connect_only_to(session, peers.values(), speedtest_current_candidate)

            # Speedtest
            print(f"Testing {speedtest_current_candidate.name}...")
            speed = speedtest_utils.speedtest()
            print(f"{speedtest_current_candidate.name} (Down: {speedtest_utils.format_bps(speed)}, Ping: {speedtest_utils.sp_client.results.ping})")
            speedtest_current_candidate.set_speed(speed)

            if speed > config.get_speedtest_threshold():
                print("Speed threshold satisfied.")
                speed_satisfied = True
                break
            else:
                print("Speed threshold not satisfied. Testing another peer...")


        if not speed_satisfied:
            # All peers failed speed threshold. Go to fastest instead.
            print("Max tries reached / Out of peers. Connecting to best-effort fastest peer.")

        # Connect to the fastest peer.
        # Yes this is unnecessary for most situations, but I don't trust my coding enough plus code is cleaner this way sooo....
        fastest_peer = sorted(speedtest_candidates, key=lambda p1: p1.rx_speed, reverse=True)[0]

        print(f"Connecting to {fastest_peer.name} (Down: {speedtest_utils.format_bps(fastest_peer.rx_speed)})...")
        connect_only_to(session, peers.values(), fastest_peer)

        print("---")
        print("Applying...")
        wg_peer.wg_apply(session)


    # === FINAL APPLY ===
    print ("=== Final apply... ===")
    # These are probably unnecessary, but they seem to fix some monitoring bugs so I kept them.
    print("Restarting WireGuard...")
    output = session.post(f"{config.opnsense_api}/core/service/restart/wireguard/{config.instance_UUID}", verify=False)
    if config.debug:
        print(output.content)

    if config.get_gateway_name() != "":
        print("Resetting Gateway watcher...")
        output = session.post(f"{config.opnsense_api}/core/service/restart/dpinger/{config.get_gateway_name()}", verify=False)
        if config.debug:
            print(output.content)

    print()
    print("============")
    print("DONE!")
    print()
    print("-- SUMMARY ---------------------------------")
    print(f"AUTOMATIC:\t{config.automatic}")

    try:
        # noinspection PyStatementEffect
        speedtest_candidates # Test existence
        print("CONNECTION TYPE:\tSpeedtest")
        print("-------------------------------------------")
        print(f"SEL\tINFO\t\t\t\t\t\t\t\t\t\t\t\tBANDWIDTH")
        for o in speedtest_candidates:
            if o == fastest_peer:
                print(f"[*]\t{o.get_human_info()}\t{speedtest_utils.format_bps(o.rx_speed)}")
            else:
                print(f"[ ]\t{o.get_human_info()}\t{speedtest_utils.format_bps(o.rx_speed)}")
    except NameError:
        print("CONNECTION TYPE:\tManual")
        print(f"== CONNECTION INFO ==")
        print(selected_peer.get_human_info())